import asyncio
from json import JSONDecodeError
from urllib.parse import parse_qs, urlencode, urlparse

import httpx
from pyrogram.raw.functions.messages import RequestWebView
from pyrogram.raw.functions.users import GetFullUser
from faker import Faker

from embykeeper.runinfo import RunStatus
from embykeeper.utils import remove_prefix, get_proxy_str, show_exception
from embykeeper.config import config

from ..link import Link
from . import BotCheckin


class NebulaCheckin(BotCheckin):
    name = "Nebula"
    bot_username = "Nebula_Account_bot"
    max_retries = 1

    async def send_checkin(self, **kw):
        bot_peer = await self.client.resolve_peer(self.bot_username)
        user_full = await self.client.invoke(GetFullUser(id=bot_peer))
        url = user_full.full_user.bot_info.menu_button.url
        url_auth = (
            await self.client.invoke(RequestWebView(peer=bot_peer, bot=bot_peer, platform="ios", url=url))
        ).url
        self.log.debug(f"请求面板: {url_auth}")
        scheme = urlparse(url_auth)
        data = remove_prefix(scheme.fragment, "tgWebAppData=")
        url_base = scheme._replace(path="/api/proxy/userCheckIn", query=f"data={data}", fragment="").geturl()
        scheme = urlparse(url_base)
        query = parse_qs(scheme.query, keep_blank_values=True)
        query = {k: v for k, v in query.items() if not k.startswith("tgWebApp")}
        token = await Link(self.client).captcha("nebula")
        if not token:
            self.log.warning("签到失败: 无法获得验证码.")
            return await self.fail(message="验证码获取失败")
        useragent = Faker().safari()
        query["token"] = token
        url_checkin = scheme._replace(query=urlencode(query, True)).geturl()
        proxy = get_proxy_str(config.proxy)
        try:
            async with httpx.AsyncClient(http2=True, proxy=proxy) as client:
                resp = await client.get(url_checkin, headers={"User-Agent": useragent})
                results = resp.json()
                message = results["message"]
                if any(s in message for s in ("未找到用户", "权限错误")):
                    self.log.info("签到失败: 账户错误.")
                    return await self.fail(message="账户错误")
                if "失败" in message:
                    self.log.info("签到失败.")
                    return await self.retry()
                if "已经" in message:
                    self.log.info("今日已经签到过了.")
                    return await self.finish(RunStatus.NONEED, "今日已签到")
                elif "成功" in message:
                    self.log.info(
                        f"[yellow]签到成功[/]: + {results['data']['get_credit']} 分 -> {results['data']['credit']} 分."
                    )
                    return await self.finish(RunStatus.SUCCESS, "签到成功")
                else:
                    self.log.warning(f"接收到异常返回信息: {message}")
                    return await self.retry()
        except (httpx.HTTPError, OSError, JSONDecodeError) as e:
            self.log.info(f"签到失败: 无法连接签到页面 ({e.__class__.__name__}).")
            show_exception(e)
            return await self.retry()



from datetime import datetime, timedelta
import random
import re

import asyncio
from typing import List
from cachetools import TTLCache
from pyrogram.types import Message, User, Chat
from pyrogram.enums import ChatMemberStatus, MessageServiceType, MessageEntityType
from pyrogram.errors import BadRequest

from ..lock import pornemby_alert, pornemby_messager_mids
from . import Monitor

__ignore__ = True


class PornembyAlertMonitor(Monitor):
    name = "Pornemby 风险急停监控"
    chat_name = "Pornemby"
    additional_auth = ["pornemby_pack"]
    allow_edit = True
    debug_no_log = True
    trigger_interval = 0
    trigger_sem = None

    user_alert_keywords = ["脚本", "真人", "admin", "全是", "举报", "每次", "机器人", "report"]
    admin_alert_keywords = ["不要", "封", "ban", "warn", "踢", "抓"]
    alert_reply_keywords = ["真人", "脚本", "每次", "在吗", "机器", "封", "warn", "ban", "回", "说"]
    alert_reply_except_keywords = ["不要回复", "别回复", "勿回复"]
    reply_words = ["?" * (i + 1) for i in range(3)] + ["嗯?", "欸?", "🤔"]
    reply_interval = 7200

    async def init(self):
        self.lock = asyncio.Lock()
        self.last_reply = None
        self.alert_remaining = 0.0
        self.member_status_cache = TTLCache(maxsize=128, ttl=86400)
        self.member_status_cache_lock = asyncio.Lock()
        self.monitor_task = asyncio.create_task(self.monitor())
        self.pin_checked = False
        self.pin_checked_lock = False
        return True

    async def check_admin(self, chat: Chat, user: User):
        if not user:
            return True
        if user.is_bot:
            return False
        async with self.member_status_cache_lock:
            if not user.id in self.member_status_cache:
                try:
                    member = await self.client.get_chat_member(chat.id, user.id)
                    self.member_status_cache[user.id] = member.status
                except BadRequest:
                    return False
        if self.member_status_cache[user.id] in (ChatMemberStatus.ADMINISTRATOR, ChatMemberStatus.OWNER):
            return True

    def check_keyword(self, message: Message, keywords: List[str]):
        content = message.text or message.caption
        if content:
            for k in keywords:
                match = re.search(k, content)
                if match:
                    return match.group(0)

    async def monitor(self):
        while True:
            await self.lock.acquire()
            while self.alert_remaining > 0:
                pornemby_alert[self.client.me.id] = True
                t = datetime.now()
                self.lock.release()
                await asyncio.sleep(1)
                await self.lock.acquire()
                self.alert_remaining -= (datetime.now() - t).total_seconds()
            else:
                pornemby_alert[self.client.me.id] = False
            self.lock.release()
            await asyncio.sleep(1)

    async def set_alert(self, time: float = None, reason: str = None):
        if time:
            async with self.lock:
                if self.alert_remaining > time:
                    return
                else:
                    msg = f"Pornemby 风险急停被触发, 停止操作 {time} 秒"
                    if reason:
                        msg += f" (原因: {reason})"
                    msg += "."
                    self.log.warning(msg)
                    self.alert_remaining = time
        else:
            msg = "Pornemby 风险急停被触发, 所有操作永久停止"
            if reason:
                msg += f" (原因: {reason})"
            msg += "."
            self.log.bind(msg=True).error(msg)
            async with self.lock:
                self.alert_remaining = float("inf")

    async def on_trigger(self, message: Message, key, reply):
        # 管理员回复水群消息, 永久停止, 若存在关键词即回复
        # 用户回复水群消息, 停止 3600 秒, 若存在关键词即回复
        if message.reply_to_message_id and message.reply_to_message_id in pornemby_messager_mids.get(
            self.client.me.id, []
        ):
            if await self.check_admin(message.chat, message.from_user):
                await self.set_alert(reason="管理员回复了水群消息")
            else:
                await self.set_alert(3600, reason="非管理员回复了水群消息")
            if self.check_keyword(message, self.alert_reply_keywords):
                if not self.check_keyword(message, self.alert_reply_except_keywords):
                    if (not self.last_reply) or (
                        self.last_reply < datetime.now() - timedelta(seconds=self.reply_interval)
                    ):
                        await asyncio.sleep(random.uniform(5, 15))
                        await message.reply(random.choice(self.reply_words))
                        self.last_reply = datetime.now()
            return

        # 管理员 @ 当前用户, 永久停止
        # 非管理员 @ 当前用户, 停止 3600 秒
        if message.entities:
            for entity in message.entities:
                if (
                    entity.type == MessageEntityType.MENTION
                    and entity.user
                    and entity.user.id == self.client.me.id
                ):
                    if await self.check_admin(message.chat, message.from_user):
                        await self.set_alert(reason="管理员 @ 了当前用户")
                    else:
                        await self.set_alert(3600, reason="非管理员 @ 了当前用户")
                    return

        # 新置顶消息包含关键词, 停止 86400 秒
        if message.service == MessageServiceType.PINNED_MESSAGE:
            keyword = self.check_keyword(
                message.pinned_message, self.user_alert_keywords + self.admin_alert_keywords
            )
            if keyword:
                await self.set_alert(86400, reason=f'有新消息被置顶, 且包含风险关键词: "{keyword}"')
            else:
                await self.set_alert(3600, reason="有新消息被置顶")
            return

        # 管理员发送消息包含关键词, 停止 86400 秒
        # 用户发送消息包含关键词, 停止 1800 秒
        keyword = self.check_keyword(message, self.user_alert_keywords + self.admin_alert_keywords)
        if keyword:
            if await self.check_admin(message.chat, message.from_user):
                await self.set_alert(86400, reason=f'管理员发送了消息, 且包含风险关键词: "{keyword}"')



from __future__ import annotations

import asyncio
from typing import Dict, List, Set, Type

from loguru import logger

from embykeeper.schema import TelegramAccount
from embykeeper.config import config
from embykeeper.runinfo import RunContext

from .messager import Messager
from .dynamic import extract, get_cls, get_names
from .link import Link
from .session import ClientsSession
from .pyrogram import Client

logger = logger.bind(scheme="telechecker")


class MessageManager:
    """消息管理器"""

    def __init__(self):
        self._tasks: Dict[str, asyncio.Task] = {}  # phone -> task
        self._running: Set[str] = set()  # Currently running phones

        # Set up config change callbacks
        config.on_list_change("telegram.account", self._handle_account_change)

    def _handle_account_change(self, added: List[TelegramAccount], removed: List[TelegramAccount]):
        """Handle account additions and removals"""
        for account in removed:
            self.stop_account(account.phone)

        for account in added:
            if account.messager and account.enabled:
                self.start_account(account)

    def stop_account(self, phone: str):
        """Stop running tasks for an account"""
        if phone in self._tasks:
            self._tasks[phone].cancel()
            del self._tasks[phone]

        self._running.discard(phone)

    def start_account(self, account: TelegramAccount):
        """Start messaging for an account"""
        if not account.messager or account.phone in self._running:
            return
        task = asyncio.create_task(self.run_account(account))
        self._tasks[account.phone] = task
        return task

    async def run_account(self, account: TelegramAccount):
        """Run messager for a single account"""
        if account.phone in self._running:
            logger.warning(f"账户 {account.phone} 的自动水群已经在执行.")
            return

        account_ctx = RunContext.get_or_create(f"messager.account.{account.phone}")

        self._running.add(account.phone)
        try:
            async with ClientsSession([account]) as clients:
                async for a, client in clients:
                    await RunContext.run(
                        lambda c: self._run_account(c, a, client),
                        description=f"{account.phone} 账号自动水群",
                        parent_ids=[account_ctx.id],
                    )
        finally:
            self._running.discard(account.phone)

    async def _run_account(self, ctx: RunContext, account: TelegramAccount, client: Client):
        """Run messagers for a single user"""
        log = logger.bind(username=client.me.name)

        # Get messager classes based on account config or global config
        site = None
        if account.site and account.site.messager is not None:
            site = account.site.messager
        elif config.site and config.site.messager is not None:
            site = config.site.messager
        else:
            site = get_names("messager")

        clses: List[Type[Messager]] = extract(get_cls("messager", names=site))

        if not clses:
            if site is not None:  # Only show warning if sites were specified but none were valid
                log.warning("没有任何有效自动水群站点, 自动水群将跳过.")
            return

        if not await Link(client).auth("messager", log_func=log.error):
            return

        messagers = []
        names = []

        for cls in clses:
            site_name = cls.__module__.rsplit(".", 1)[-1]
            site_ctx = RunContext.prepare(f"{site_name} 站点自动水群", parent_ids=ctx.id)
            messager = cls(
                account=account,
                me=client.me,
                context=site_ctx,
                config=config.messager.get_site_config(site_name),
            )
            messagers.append(messager)
            names.append(messager.name)

        if names:
            log.debug(f'已启用自动水群器: {", ".join(names)}')

        # Start all messagers concurrently
        await asyncio.gather(*[m._start() for m in messagers])

    async def run_all(self):
        """Run messaging for all enabled accounts"""
        accounts = [a for a in config.telegram.account if a.enabled and a.messager]
        tasks = []
        for account in accounts:
            task = self.start_account(account)
            if task:  # start_account might return None if account is already running
                tasks.append(task)
        if tasks:
            await asyncio.gather(*tasks)



import asyncio
import random
from typing import Iterable
from loguru import logger

from embykeeper.config import config
from embykeeper.schema import SubsonicAccount
from embykeeper.utils import show_exception

from .api import Subsonic, SubsonicConnectError, SubsonicRequestError

logger = logger.bind(scheme="subsonic")


class SubsonicPlayer:
    def __init__(self, account: SubsonicAccount):
        self.a = account
        self.cf_clearance = None
        self.useragent = None

        self.log = logger.bind(server=self.a.name or self.hostname, username=self.a.username)

    @property
    def proxy(self):
        return config.proxy if self.a.use_proxy else None

    @property
    def hostname(self):
        return self.a.url.host

    async def login(self):
        """登录账号."""
        client = Subsonic(
            server=str(self.a.url),
            username=self.a.username,
            password=self.a.password,
            proxy=self.proxy,
            useragent=self.useragent,
            client=self.a.client,
            version=self.a.client_version,
        )
        try:
            info = await client.ping()
            if info.is_ok:
                self.log.info(
                    f'成功连接至服务器 ({(info.type or "unknown").capitalize()} {info.version or "X.X"}).'
                )
                return client
            else:
                self.log.error(f"服务器登陆错误, 请重新检查配置: {info.error_message}")
                return None
        except SubsonicConnectError as e:
            self.log.warning(f"服务器登陆错误, 无法连接: {e}")
            return None
        except SubsonicRequestError as e:
            self.log.warning(f"服务器登陆错误, 服务器异常: {e}")
            return None

    async def play(self, client: Subsonic):
        """模拟连续播放音频直到达到指定总时长."""

        try:
            if isinstance(self.a.time, Iterable):
                req_time = random.uniform(*self.a.time)
            else:
                req_time = self.a.time
        except TypeError:
            self.log.warning(f"无法解析 time 配置, 请检查配置: {self.a.time} (应该为数字或两个数字的数组).")
            return False

        played_time = 0
        retry = 0

        while played_time < req_time:
            try:
                songs = await client.get_random_songs()
                if not songs:
                    self.log.warning("未能获取到任何歌曲.")
                    return False
                song = random.choice(songs)
                song_id = song.get("id", None)
                if not song_id:
                    self.log.warning("获取到歌曲信息不完整, 正在重试.")
                    continue

                song_title = song.get("title", "未知歌曲")
                song_duration = float(song.get("duration", 60))
                remaining_time = req_time - played_time

                play_duration = min(remaining_time, song_duration) if song_duration > 0 else remaining_time

                self.log.info(f'开始播放 "{song_title}", 剩余时间 {remaining_time:.0f} 秒.')
                while retry < config.subsonic.retries:
                    try:
                        await client.scrobble(song_id, submission=False)
                        await asyncio.wait_for(client.stream_noreturn(song_id), timeout=play_duration)
                        played_time += play_duration
                        await client.scrobble(song_id, submission=True)
                        self.log.info(f'完成播放 "{song_title}", 已播放 {played_time:.0f} 秒.')
                        retry = 0
                        break
                    except asyncio.TimeoutError:
                        # 正常超时，说明歌曲播放完成
                        played_time += play_duration
                        await client.scrobble(song_id, submission=True)
                        self.log.info(f'完成播放 "{song_title}", 已播放 {played_time:.0f} 秒.')
                        break
                    except Exception as e:
                        retry += 1
                        if retry >= config.subsonic.retries:
                            self.log.error(f"播放出错且达到最大重试次数, 停止播放.")
                            show_exception(e, regular=False)
                            return False
                        self.log.warning(f"播放出错 (重试 {retry}/{config.subsonic.retries}), 正在重试.")
                        show_exception(e, regular=False)
                        await asyncio.sleep(1)
                        continue
            except Exception as e:
                retry += 1
                if retry >= config.subsonic.retries:
                    self.log.error(f"播放出错且达到最大重试次数, 停止播放.")
                    show_exception(e, regular=False)
                    return False
                self.log.warning(f"访问出错 (重试 {retry}/{config.subsonic.retries}), 正在重试: {e}.")
                show_exception(e, regular=True)
                await asyncio.sleep(1)
                continue
        return True



import random
from pyrogram.types import Message, InlineKeyboardMarkup
from pyrogram.errors import RPCError
from pyrogram.enums import MessageEntityType

from embykeeper.utils import flatten

from ..lock import pornemby_alert
from . import Monitor


class PornembyDragonRainMonitor:
    class PornembyDragonRainClickMonitor(Monitor):
        name = "Pornemby 红包雨"
        chat_user = ["PronembyTGBot2_bot", "PronembyTGBot3_bot", "PornembyBot", "Porn_Emby_Bot"]
        chat_name = "Pornemby"
        chat_keyword = [None]
        additional_auth = ["pornemby_pack"]
        allow_edit = True
        debug_no_log = True

        async def on_trigger(self, message: Message, key, reply):
            if pornemby_alert.get(self.client.me.id, False):
                self.log.info(f"由于风险急停不抢红包.")
                return
            if message.reply_markup:
                if isinstance(message.reply_markup, InlineKeyboardMarkup):
                    buttons = flatten(message.reply_markup.inline_keyboard)
                    for b in buttons:
                        if "红包奖励" in b.text:
                            if random.random() > self.config.get("possibility", 1.0):
                                self.log.info(f"由于概率设置不抢红包.")
                                return
                            try:
                                await message.click(b.text)
                            except (TimeoutError, RPCError):
                                self.log.info("检测到 Pornemby 抢红包雨, 已点击抢红包, 等待结果.")
                            else:
                                self.log.info("检测到 Pornemby 抢红包雨, 已点击抢红包, 等待结果.")
                            return

    class PornembyDragonRainStatusMonitor(Monitor):
        name = "Pornemby 红包雨结果"
        chat_user = ["PronembyTGBot2_bot", "PronembyTGBot3_bot", "PornembyBot", "Porn_Emby_Bot"]
        chat_name = "Pornemby"
        chat_keyword = r"恭喜\s+(.*):本次获得(\d+)豆"
        allow_edit = True

        async def on_trigger(self, message: Message, key, reply):
            for me in message.entities:
                if me.type == MessageEntityType.TEXT_MENTION:
                    if me.user.id == self.client.me.id:
                        self.log.info(f"红包雨结果: 恭喜获得 {key[1]} 豆.")



from .pornemby_exam import _PornembyExamAnswerMonitor

__ignore__ = True


class TestPornembyExamMonitor(_PornembyExamAnswerMonitor):
    name = "Pornemby 科举回答测试"
    chat_name = "api_group"
    chat_user = ["embykeeper_test_bot"]



import asyncio
import random
from urllib.parse import parse_qs, urlparse

from pyrogram import filters
from pyrogram.types import Message
from pyrogram.raw.functions.messages import RequestWebView
from pyrogram.errors import MessageIdInvalid
import httpx
from faker import Faker

from embykeeper.config import config
from embykeeper.utils import get_proxy_str

from ..link import Link
from . import Monitor


class FutureMonitor(Monitor):
    name = "未响"
    chat_keyword = r"FutureEcho-register-\w{8}-\w{4}-\w{3}-\w{5}"
    bot_username = "lotayu_bot"
    chat_name = "FutureEcho_Chat"
    notify_create_name = True
    allow_edit = False
    additional_auth = ["captcha"]

    async def solve_captcha(self, url: str):
        token = await Link(self.client).captcha("future_echo")
        if not token:
            return False
        else:
            scheme = urlparse(url)
            params = parse_qs(scheme.query)
            url_submit = scheme._replace(path="/x/api/submit", query="", fragment="").geturl()
            uuid = params.get("id", [None])[0]
            origin = scheme._replace(path="/", query="", fragment="").geturl()
            useragent = Faker().safari()
            headers = {
                "Content-Type": "application/x-www-form-urlencoded",
                "Referer": url,
                "Origin": origin,
                "User-Agent": useragent,
            }
            data = {
                "uuid": uuid,
                "cf-turnstile-response": token,
            }
            try:
                async with httpx.AsyncClient(http2=True, proxy=get_proxy_str(config.proxy)) as client:
                    resp = await client.post(url_submit, headers=headers, data=data)
                    result = resp.text
                    if "完成" in result:
                        return True
            except:
                return False

    async def on_trigger(self, message: Message, key, reply):
        for i in range(3):
            if i:
                self.log.info(f"正在重试注册 ({i}/3).")
            msg = await self.client.wait_reply(self.bot_username, f"/start")
            text = msg.text or msg.caption
            if "你还未加入" in text:
                self.log.error("账户错误, 无法注册.")
                return
            async with self.client.catch_reply(self.bot_username) as f1:
                async with self.client.catch_edit(msg, ~filters.regex("请先完成验证")) as f2:
                    try:
                        msg = await msg.click("💡註冊帳戶", timeout=1)
                    except (TimeoutError, MessageIdInvalid):
                        pass
                    except ValueError:
                        self.log.error("未能找到注册按钮, 无法注册.")
                        return
                    try:
                        done, pending = await asyncio.wait(
                            [f1, f2], return_when=asyncio.FIRST_COMPLETED, timeout=10
                        )
                    except asyncio.TimeoutError:
                        self.log.error("点击注册按钮无响应, 无法注册.")
                        return
                    else:
                        for f in pending:
                            f.cancel()
                        msg = list(done)[0].result()
            text = msg.text or msg.caption
            if "验证您的身份" in text:
                self.log.info("需要验证身份, 正在解析.")
                url = None
                if msg.reply_markup:
                    buttons = [button for line in msg.reply_markup.inline_keyboard for button in line]
                    for b in buttons:
                        if "Verify" in b.text and b.web_app:
                            url = b.web_app.url
                            break
                if not url:
                    self.log.error("需要验证身份但没有找到 URL, 无法注册.")
                    return
                bot_peer = await self.client.resolve_peer(self.bot_username)
                url_auth = (
                    await self.client.invoke(
                        RequestWebView(peer=bot_peer, bot=bot_peer, platform="ios", url=url)
                    )
                ).url
                if not await self.solve_captcha(url_auth):
                    self.log.error("验证码解析失败, 无法注册.")
                    return
                else:
                    await asyncio.sleep(random.uniform(3, 5))
                    self.log.info("已成功验证, 重新进行注册流程.")
                    continue
            else:
                if ("邀請碼" in text) or ("邀请码" in text):
                    msg = await self.client.wait_reply(self.bot_username, key)
                    text = msg.text or msg.caption
                    if "无效" in text:
                        self.log.error("邀请码无效, 无法注册.")
                        return
                if "用户名" in text:
                    msg = await self.client.wait_reply(self.bot_username, self.unique_name)
                    text = msg.text or msg.caption
                if "邮箱地址" in text:
                    msg = await self.client.wait_reply(self.bot_username, f"{self.unique_name}@gmail.com")
                    text = msg.text or msg.caption
                if "创建成功" in text:
                    self.log.bind(msg=True).info(f"已在 Bot @{self.bot_username} 成功创建用户, 请查看.")
                    return
                else:
                    self.log.warning(f"注册失败, 可能是注册流程出错, 请检查.")
                    return



from . import Messager, MessageSchedule

__ignore__ = True


class MarmotMessager(Messager):
    name = "Marmot"
    chat_name = -1001975531465
    default_messages = [
        MessageSchedule("goodday-wl@v1.yaml", possibility=0.4, only="weekends"),
        MessageSchedule("goodnoon-wl@v2.yaml", possibility=0.1, only="weekends"),
        MessageSchedule("goodeve-wl@v1.yaml", possibility=0.1, only="weekends"),
        MessageSchedule("goodnight-wl@v1.yaml", possibility=0.4, only="weekends"),
    ]



from typing import List, Optional, Union, Dict, Any, ClassVar
from pydantic import BaseModel, Field, model_validator, ValidationError
from pydantic.networks import HttpUrl

DEFAULT_TIME_RANGE = "<11:00AM,11:00PM>"
DEFAULT_EMBY_INTERVAL_DAYS = "<7,12>"


class ConfigModel(BaseModel):
    model_config = {"extra": "forbid"}

    @model_validator(mode="before")
    @classmethod
    def validate_extra_fields(cls, values):
        if not isinstance(values, dict):
            return values
        if cls.model_config.get("extra") == "allow":
            return values
        allowed_fields = set(cls.model_fields.keys())
        extra_fields = set(values.keys()) - allowed_fields
        if extra_fields:
            raise ValueError(
                f"包含未知设置项：{', '.join(extra_fields)}, 允许的设置项: {', '.join(allowed_fields)}"
            )
        return values


class UseStr(str):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, info):
        if isinstance(v, (int, float)):
            return str(v)
        return v


class UseHttpUrl(HttpUrl):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, info):
        if isinstance(v, str) and not v.startswith(("http://", "https://")):
            v = f"https://{v}"
        return HttpUrl(v)


class ProxyConfig(ConfigModel):
    hostname: Optional[str] = None
    port: Optional[int] = Field(None, gt=0)
    scheme: Optional[str] = Field(None, pattern="^(socks5|http)$")
    username: Optional[str] = None
    password: Optional[str] = None


class CheckinerConfig(ConfigModel):
    time_range: Optional[UseStr] = DEFAULT_TIME_RANGE
    interval_days: Optional[UseStr] = "1"
    timeout: Optional[int] = 120
    retries: Optional[int] = 4
    concurrency: Optional[int] = 1
    random_start: Optional[int] = 60

    model_config = {"extra": "allow"}

    def get_site_config(self, site: str) -> Dict[str, Any]:
        return getattr(self, site, {})


class MonitorConfig(ConfigModel):
    model_config = {"extra": "allow"}

    def get_site_config(self, site: str) -> Dict[str, Any]:
        return getattr(self, site, {})


class MessagerConfig(ConfigModel):
    model_config = {"extra": "allow"}

    def get_site_config(self, site: str) -> Dict[str, Any]:
        return getattr(self, site, {})


class NotifierConfig(ConfigModel):
    enabled: Optional[bool] = False
    account: Optional[Union[int, str]] = 1
    immediately: Optional[bool] = False
    once: Optional[bool] = False


class SiteConfig(ConfigModel):
    checkiner: Optional[List[str]] = None
    monitor: Optional[List[str]] = None
    messager: Optional[List[str]] = None


class MediaServerBaseConfig(ConfigModel):
    time_range: Optional[UseStr] = DEFAULT_TIME_RANGE
    interval_days: Optional[UseStr] = DEFAULT_EMBY_INTERVAL_DAYS
    concurrency: Optional[int] = 1
    retries: Optional[int] = 5


class EmbyAccount(ConfigModel):
    url: UseHttpUrl
    username: str
    password: str
    name: str = None
    time: Optional[Union[int, List[int]]] = [300, 600]
    continuous: Optional[bool] = False
    useragent: Optional[str] = None
    client: Optional[str] = None
    client_version: Optional[str] = None
    device: Optional[str] = None
    device_id: Optional[str] = None
    allow_multiple: Optional[bool] = True
    allow_stream: Optional[bool] = False
    cf_challenge: Optional[bool] = True
    use_proxy: Optional[bool] = True
    play_id: Optional[str] = None
    enabled: Optional[bool] = True

    # 站点单独配置
    interval_days: Optional[Union[int, str]] = None
    time_range: Optional[str] = None

    # 向后兼容字段
    interval: Optional[Union[int, str]] = None
    watchtime: Optional[str] = None
    hide: Optional[bool] = None
    ua: Optional[str] = None
    jellyfin: Optional[bool] = None


class EmbyConfig(MediaServerBaseConfig):
    account: Optional[List[EmbyAccount]] = []


class SubsonicAccount(ConfigModel):
    url: UseHttpUrl
    username: str
    password: str
    name: str = None
    time: Optional[Union[int, List[int]]] = None
    useragent: Optional[str] = None
    client: Optional[str] = None
    client_version: Optional[str] = None
    use_proxy: Optional[bool] = True
    enabled: Optional[bool] = True

    # 站点单独配置
    interval_days: Optional[Union[int, str]] = None
    time_range: Optional[str] = None

    # 向后兼容字段
    ua: Optional[str] = None
    version: Optional[str] = None


class SubsonicConfig(MediaServerBaseConfig):
    account: Optional[List[SubsonicAccount]] = []


class TelegramAccount(ConfigModel):
    phone: str = Field(description="Telegram phone number")

    @model_validator(mode="before")
    @classmethod
    def clean_phone(cls, values):
        if isinstance(values, dict) and "phone" in values:
            values["phone"] = values["phone"].replace(" ", "")
        return values

    checkiner: Optional[bool] = True
    monitor: Optional[bool] = False
    messager: Optional[bool] = False
    api_id: Optional[str] = None
    api_hash: Optional[str] = None
    session: Optional[str] = None
    enabled: Optional[bool] = True

    # 账号单独配置
    site: Optional[SiteConfig] = None
    checkiner_config: Optional[CheckinerConfig] = None

    def get_config_key(self):
        import hashlib

        unique_str = f"{self.phone}:{self.api_id or ''}:{self.api_hash or ''}"
        hash_value = hashlib.sha256(unique_str.encode()).hexdigest()[:8]
        return f"{self.phone}/{hash_value}"


class TelegramConfig(ConfigModel):
    account: Optional[List[TelegramAccount]] = []


class BotConfig(ConfigModel):
    token: str


class Config(ConfigModel):
    alias_map: ClassVar[Dict[str, str]] = {
        "emby.time_range": "watchtime",
        "emby.concurrency": "watch_concurrent",
        "subsonic.time_range": "listentime",
        "subsonic.concurrency": "listen_concurrent",
        "checkiner.time_range": "time",
        "checkiner.timeout": "timeout",
        "checkiner.retries": "retries",
        "checkiner.concurrency": "concurrent",
        "checkiner.random_start": "random",
        "emby.interval_days": "interval",
        "subsonic.interval_days": "interval",
        "site": "service",
    }

    mongodb: Optional[str] = None
    basedir: Optional[str] = None
    nofail: Optional[bool] = True
    debug_cron: Optional[bool] = False
    proxy: Optional[ProxyConfig] = None
    emby: Optional[EmbyConfig] = EmbyConfig()
    subsonic: Optional[SubsonicConfig] = SubsonicConfig()
    checkiner: Optional[CheckinerConfig] = CheckinerConfig()
    monitor: Optional[MonitorConfig] = MonitorConfig()
    messager: Optional[MessagerConfig] = MessagerConfig()
    telegram: Optional[TelegramConfig] = TelegramConfig()
    notifier: Optional[NotifierConfig] = NotifierConfig()
    site: Optional[SiteConfig] = None

    # 向后兼容字段
    time: Optional[str] = None
    watchtime: Optional[str] = None
    listentime: Optional[str] = None
    interval: Optional[Union[int, str]] = None
    timeout: Optional[int] = None
    retries: Optional[int] = None
    concurrent: Optional[int] = None
    watch_concurrent: Optional[int] = None
    listen_concurrent: Optional[int] = None
    random: Optional[int] = None
    notify_immediately: Optional[bool] = None
    service: Optional[SiteConfig] = None

    # 调试字段
    bot: Optional[BotConfig] = None

    @model_validator(mode="before")
    @classmethod
    def handle_aliases(cls, values: Dict[str, Any]) -> Dict[str, Any]:
        for service in ["emby", "subsonic", "telegram"]:
            if service in values and isinstance(values[service], list):
                if service == "telegram":
                    # Convert telegram account fields
                    for account in values[service]:
                        if "send" in account:
                            account["messager"] = account.pop("send")
                        if "checkin" in account:
                            account["checkiner"] = account.pop("checkin")
                if service == "emby":
                    # Convert emby account fields
                    for account in values[service]:
                        if "ua" in account:
                            account["useragent"] = account.pop("ua")
                if service == "subsonic":
                    # Convert subsonic account fields
                    for account in values[service]:
                        if "ua" in account:
                            account["useragent"] = account.pop("ua")
                        if "version" in account:
                            account["client_version"] = account.pop("version")
                values[service] = {"account": values[service]}

        if "notifier" in values:
            notifier_value = values["notifier"]
            if isinstance(notifier_value, str):
                values["notifier"] = {
                    "enabled": True,
                    "account": notifier_value,
                }
            elif isinstance(notifier_value, bool):
                values["notifier"] = {
                    "enabled": notifier_value,
                }
            elif isinstance(notifier_value, int):
                values["notifier"] = {
                    "enabled": notifier_value > 0,
                    "account": notifier_value,
                }

        for new_field, old_field in cls.alias_map.items():
            if old_field in values and values[old_field] is not None:
                parts = new_field.split(".")
                target = values
                for part in parts[:-1]:
                    target.setdefault(part, {})
                    target = target[part]
                target[parts[-1]] = values[old_field]

        return values


def format_errors(e: ValidationError) -> str:
    """自定义错误信息格式化"""

    error_translations = {
        "Input should be a valid boolean": "输入应为布尔值 (true/false)",
        "Input should be a valid integer": "输入应为有效的整数",
        "Input should be a valid string": "输入应为有效的字符串, 用英文双引号包裹",
        "Input should be a valid list": "输入应为有效的列表, 用[]符号包裹",
        "Input should be a valid URL": "输入应为有效的URL地址",
        "Field required": "必填字段",
        "Value error": "配置验证错误",
        "Input should match pattern": "输入格式不匹配要求",
        "Value is not a valid dict": "输入应为有效的字典格式",
    }

    reverse_aliases = {}
    for new_field, old_field in Config.alias_map.items():
        if old_field not in reverse_aliases:
            reverse_aliases[old_field] = []
        reverse_aliases[old_field].append(new_field)

    error_groups = {}
    error_messages = ["配置文件错误，请检查配置文件:"]

    for error in e.errors():
        location = list(error["loc"])
        msg = error["msg"]

        # 翻译错误消息
        for eng, chn in error_translations.items():
            if callable(chn):
                msg = msg.replace(eng, chn(error["loc"]))
            else:
                msg = msg.replace(eng, chn)

        # 如果是根级别的错误，直接添加错误信息
        if not location:
            error_messages.append(f"  {msg}")
            continue

        loc_str = " -> ".join(str(loc) for loc in location)

        error_key = (() if len(location) <= 1 else tuple(location[1:])) + (msg,)

        # 检查是否有相关的别名字段
        if location[0] in reverse_aliases:
            for new_field in reverse_aliases[location[0]]:
                new_loc = new_field.split(".")
                if len(location) > 1:
                    new_loc.extend(location[1:])
                new_loc_str = " -> ".join(new_loc)
                group_key = f"  {new_loc_str}\n  (旧版本为: {loc_str})"
                error_groups[error_key] = (group_key, msg)
        else:
            error_groups[error_key] = (f"  {loc_str}", msg)

    # 添加分组后的错误消息
    for _, (location, msg) in error_groups.items():
        error_messages.append(f"{location}:")
        error_messages.append(f"    {msg}")

    error_messages.append("详细说明请访问: https://emby-keeper.github.io/guide/配置文件")
    return "\n".join(error_messages)


if __name__ == "__main__":
    import sys
    import tomli
    import pprint

    if len(sys.argv) < 2:
        print("Usage: python schema.py <config.toml>")
        sys.exit(1)

    try:
        with open(sys.argv[1], "rb") as f:
            config_dict = tomli.load(f)
        config = Config(**config_dict)
        pprint.pprint(config.model_dump())
    except FileNotFoundError:
        print(f"错误: 配置文件 '{sys.argv[1]}' 未找到")
        sys.exit(1)
    except tomli.TOMLDecodeError as e:
        print(f"错误: TOML格式无效 - {e}")
        sys.exit(1)
    except ValidationError as e:
        print(format_errors(e))
        sys.exit(1)



from . import BotCheckin

__ignore__ = True


class DogSGKCheckin(BotCheckin):
    name = "狗狗社工库"
    bot_username = "DogeSGK_bot"
    bot_checkin_cmd = "/sign"
    bot_success_pat = r".*\+(\d+)"
    bot_checked_keywords = "请勿重复签到"



import asyncio
import random
from typing import List
from urllib.parse import parse_qs, urlparse

from loguru import logger

from embykeeper.config import config
from embykeeper.schedule import Scheduler
from embykeeper.utils import show_exception, truncate_str
from embykeeper.runinfo import RunContext, RunStatus
from embykeeper.var import console
from embykeeper.schema import EmbyAccount

from .api import Emby, EmbyPlayError, EmbyConnectError, EmbyRequestError, EmbyError

logger = logger.bind(scheme="embywatcher")


class EmbyManager:
    async def play_url(self, url: str):
        parsed = urlparse(url)

        fragment_parts = parsed.fragment.split("?", 1)
        if len(fragment_parts) > 1:
            params = parse_qs(fragment_parts[1])
        else:
            params = {}

        if not params.get("id"):
            logger.error(
                "无效的 URL 格式, 无法解析视频 ID. 应为类似:\nhttps://example.com/web/#/details?id=xxx&serverId=xxx"
            )
            return False

        iid = params["id"][0]

        # 在config中查找匹配的emby配置
        account = None
        for a in config.emby.account:
            if a.url.host == parsed.netloc:
                account = a
                break

        if not account:
            logger.error(f"在配置中未找到匹配的 Emby 服务器: {parsed.netloc}")
            return False

        ctx = RunContext.prepare(description="播放指定 URL 视频")
        ctx.start(RunStatus.INITIALIZING)

        emby = Emby(account)
        try:
            if not await emby.login():
                return ctx.finish(RunStatus.FAIL, "登陆失败")
            emby.log.info("使用以下 Headers:")
            console.rule("Headers")
            headers = emby.build_headers()
            for k, v in headers.items():
                console.print(f"{k.title()}: {v}")
            console.rule()
            item = await emby.get_item(iid)
            if not item:
                raise ValueError(f"无法找到 ID 为 {iid} 的视频")
            name = truncate_str(item.get("Name", "(未命名视频)"), 10)
            emby.log.info(f'10 秒后, 将开始播放该视频 300 秒: "{name}"')
            await asyncio.sleep(1)
            emby.log.info(f'开始播放视频 300 秒: "{name}"')
            try:
                await emby.play(item, time=300)
            except EmbyPlayError as e:
                emby.log.error(f"播放失败: {e}")
                return ctx.finish(RunStatus.FAIL, "播放失败")
            return ctx.finish(RunStatus.SUCCESS, "播放成功")
        except EmbyConnectError as e:
            if emby.proxy:
                emby.log.error(f"无法连接到服务器, 可能是您的代理服务器设置错误或无法连通: {e}")
            else:
                emby.log.error(f"无法连接到服务器, 可能是您没有使用代理: {e}")
            return ctx.finish(RunStatus.FAIL, "连接失败")
        except EmbyRequestError as e:
            emby.log.error(f"服务器异常: {e}")
            return ctx.finish(RunStatus.FAIL, "服务器异常")
        except Exception as e:
            emby.log.error("播放视频时发生错误, 播放失败.")
            show_exception(e, regular=False)
            return ctx.finish(RunStatus.ERROR, "异常错误")

    def get_spec(self, a: EmbyAccount):
        return f"{a.username}@{a.name or a.url.host}"

    async def _watch_main(self, accounts: List[EmbyAccount], instant: bool = False):
        if not accounts:
            return None
        logger.info("开始执行 Emby 保活.")
        tasks = []
        sem = asyncio.Semaphore(config.emby.concurrency or 100000)

        ctx = RunContext.prepare(description="使用全局设置的 Emby 统一保活")
        ctx.start(RunStatus.INITIALIZING)

        async def watch_wrapper(account: EmbyAccount, sem):
            async with sem:
                try:
                    emby = Emby(account)
                except Exception:
                    logger.error(f"初始化失败: {e}")
                    show_exception(e, regular=False)
                    return account, False
                if not instant:
                    wait = random.uniform(180, 360)
                    emby.log.info(f"播放视频前随机等待 {wait:.0f} 秒.")
                    await asyncio.sleep(wait)
                try:
                    emby.log.info(f"正在登陆并获取首页视频项目.")
                    if not emby.user_id:
                        if not await emby.login():
                            emby.log.warning(f"保活失败: 无法登陆.")
                            return account, False
                    await emby.load_main_page()
                    if not emby.items:
                        emby.log.warning("保活失败: 无法获取首页中的视频项目")
                        return account, False
                    else:
                        emby.log.info(f"成功登陆, 获取了 {len(emby.items)} 个首页视频项目.")
                    await asyncio.sleep(random.uniform(2, 5))
                    return account, await emby.watch()
                except EmbyError as e:
                    emby.log.warning(f"保活失败: {e}.")
                    return account, False
                except Exception as e:
                    emby.log.warning(f"保活失败: {e}")
                    show_exception(e, regular=False)
                    return account, False

        for account in accounts:
            if account.enabled:
                tasks.append(watch_wrapper(account, sem))

        failed_accounts = []
        successful_accounts = []
        results = await asyncio.gather(*tasks)
        for a, success in results:
            if success:
                successful_accounts.append(self.get_spec(a))
            else:
                failed_accounts.append(self.get_spec(a))
        fails = len(failed_accounts)

        if fails:
            if len(accounts) == 1:
                logger.error(f"保活失败: {', '.join(failed_accounts)}")
            else:
                logger.error(f"保活失败 ({fails}/{len(tasks)}): {', '.join(failed_accounts)}")
            return ctx.finish(RunStatus.FAIL, f"保活失败")
        if len(accounts) == 1:
            logger.bind(log=True).info(f"保活成功: {', '.join(successful_accounts)}.")
        else:
            logger.bind(log=True).info(
                f"保活成功 ({len(tasks)}/{len(tasks)}): {', '.join(successful_accounts)}."
            )
        return ctx.finish(RunStatus.SUCCESS, f"保活成功")

    async def run_all(self, instant: bool = False):
        return await self._watch_main(config.emby.account, instant)

    async def schedule_all(self, instant: bool = False):
        unified_accounts: List[EmbyAccount] = []
        independent_accounts: List[EmbyAccount] = []
        tasks = []

        # Separate accounts into global and site-specific
        for account in config.emby.account:
            if not account.enabled:
                continue
            if account.time_range or account.interval_days:
                independent_accounts.append(account)
            else:
                unified_accounts.append(account)

        # Schedule global accounts together
        if unified_accounts:
            on_next_time = lambda t: logger.bind(log=True).info(
                f"下一次 Emby 保活将在 {t.strftime('%m-%d %H:%M %p')} 进行."
            )
            scheduler = Scheduler.from_str(
                func=lambda ctx: self._watch_main(unified_accounts, instant),
                interval_days=config.emby.interval_days,
                time_range=config.emby.time_range,
                on_next_time=on_next_time,
                sid="emby.watch.global",
                description="Emby 保活任务",
            )
            tasks.append(scheduler.schedule())

        # Schedule individual site accounts
        for account in independent_accounts:
            account_spec = self.get_spec(account)
            time_range = account.time_range or config.emby.time_range
            interval = account.interval_days or config.emby.interval_days

            on_next_time = lambda t: logger.bind(log=True).info(
                f"下一次 Emby 账号 ({account_spec}) 的签到将在 {t.strftime('%m-%d %H:%M %p')} 进行."
            )
            scheduler = Scheduler.from_str(
                func=lambda ctx: self._watch_main([account], False),
                interval_days=interval,
                time_range=time_range,
                on_next_time=on_next_time,
                sid=f"emby.watch.{account_spec}",
                description=f"Emby 保活任务 - {account_spec}",
            )
            tasks.append(scheduler.schedule())

        if not tasks:
            logger.info("没有需要执行的 Emby 保活任务")

        await asyncio.gather(*tasks)



import asyncio
import base64
import binascii
import pickle
import random
import struct
import sys
import tempfile
from datetime import datetime, timezone
from pathlib import Path
from sqlite3 import OperationalError
from typing import List

import httpx
from loguru import logger
from pyrogram.errors import ApiIdPublishedFlood, AuthKeyDuplicated, BadMsgNotification, RPCError, Unauthorized
from pyrogram.storage.storage import Storage
from rich.prompt import Prompt

from embykeeper import __name__ as __product__, __version__, var
from embykeeper.utils import get_proxy_str, show_exception, to_iterable
from embykeeper.schema import TelegramAccount
from embykeeper.config import config
from embykeeper.cache import cache

from .pyrogram import Client, logger
from .telethon import TelethonUtils

_id = b"\x80\x04\x95\x15\x00\x00\x00\x00\x00\x00\x00]\x94(K2K3K7K8K5K8K4K6e."
_hash = b"\x80\x04\x95E\x00\x00\x00\x00\x00\x00\x00]\x94(KbKdK7K4K0KeKaK2KaKcKeKeK7K3K9K0KeK0KbK3K5K4KeKcK8K0K9KcK8K7K0Kfe."
_decode = lambda x: "".join(map(chr, to_iterable(pickle.loads(x))))

# "nicegram": {"api_id": "94575", "api_hash": "a3406de8d171bb422bb6ddf3bbd800e2"}
# "tgx-android": {"api_id": "21724", "api_hash": "3e0cb5efcd52300aec5994fdfc5bdc16"}
# "tg-react": {"api_id": "414121", "api_hash": "db09ccfc2a65e1b14a937be15bdb5d4b"}

API_ID = _decode(_id)
API_HASH = _decode(_hash)


class ClientsSession:
    pool = {}
    lock = asyncio.Lock()
    watch = None

    @classmethod
    async def watchdog(cls, timeout=120):
        logger.debug("Telegram 账号池看门狗启动.")
        try:
            counter = {}
            while True:
                await asyncio.sleep(10)
                for p in list(cls.pool):
                    try:
                        if cls.pool[p][1] <= 0:
                            if p in counter:
                                counter[p] += 1
                                if counter[p] >= timeout / 10:
                                    counter[p] = 0
                                    await cls.clean(p)
                            else:
                                counter[p] = 1
                        else:
                            counter.pop(p, None)
                    except (TypeError, KeyError):
                        pass
        except asyncio.CancelledError:
            await cls.shutdown()

    @classmethod
    async def clean(cls, phone: str):
        async with cls.lock:
            entry = cls.pool.get(phone, None)
            if not entry:
                return
            try:
                client: Client
                client, ref = entry
            except TypeError:
                return
            if not ref:
                logger.debug(f'登出账号 "{client.phone_number}".')
                await client.stop()
                cls.pool.pop(phone, None)

    @classmethod
    async def clean_all(cls):
        for phone in list(cls.pool):
            await cls.clean(phone)

    @classmethod
    async def shutdown(cls):
        """停止所有 Telegram 客户端会话."""
        # 1. 取消所有 pool 中的 task
        for v in cls.pool.values():
            if isinstance(v, asyncio.Task):
                v.cancel()

        # 2. 向所有 client 发送停止信号并登出账号
        logger.debug(f"Telegram 账号池停止, 正在停止所有账号客户端.")
        for v in cls.pool.values():
            if isinstance(v, tuple):
                client: Client = v[0]
                try:
                    logger.debug(f'登出账号 "{client.phone_number}".')
                    await client.terminate()
                except Exception as e:
                    logger.debug(f'登出账号 "{client.phone_number}" 时发生错误: {e}')

    def __init__(self, accounts: List[TelegramAccount], in_memory=False, proxy=None, basedir=None):
        self.accounts = accounts
        self.phones = []
        self.done = asyncio.Queue()
        self.in_memory = in_memory

        self._proxy = proxy
        self._basedir = basedir

        if not self.watch:
            self.__class__.watch = asyncio.create_task(self.watchdog())

    @property
    def basedir(self):
        return Path(self._basedir) if self._basedir else config.basedir

    @property
    def proxy(self):
        return self._proxy or config.proxy

    async def test_network(self):
        url = "https://telegram.org"
        proxy_str = get_proxy_str(self.proxy)
        try:
            async with httpx.AsyncClient(http2=True, proxy=proxy_str, timeout=20) as client:
                resp = await client.head(url)
                if resp.status_code == 200:
                    return True
                else:
                    logger.warning(f"检测网络状态时发生错误, 网络检测将被跳过.")
                    return False
        except httpx.ProxyError as e:
            if proxy_str:
                logger.warning(
                    f"无法连接到您的代理 ({proxy_str}), 您的网络状态可能不好, 敬请注意. 程序将继续运行."
                )
            return False
        except (httpx.ConnectError, httpx.ConnectTimeout):
            logger.warning(f"无法连接到 Telegram 服务器, 您的网络状态可能不好, 敬请注意. 程序将继续运行.")
            return False
        except Exception as e:
            logger.warning(f"检测网络状态时发生错误, 网络检测将被跳过.")
            show_exception(e)
            return False

    async def test_time(self):
        url = "https://ip.ddnspod.com/timestamp"
        proxy_str = get_proxy_str(self.proxy)
        try:
            async with httpx.AsyncClient(http2=True, proxy=proxy_str) as client:
                resp = await client.get(url)
                if resp.status_code == 200:
                    timestamp = int(resp.content.decode())
                else:
                    logger.warning(f"世界时间接口异常, 系统时间检测将跳过, 敬请注意. 程序将继续运行.")
                    return False
                nowtime = datetime.now(timezone.utc).timestamp()
                if abs(nowtime - timestamp / 1000) > 30:
                    logger.warning(
                        f"您的系统时间设置不正确, 与世界时间差距过大, 可能会导致连接失败, 敬请注意. 程序将继续运行."
                    )
        except httpx.HTTPError:
            logger.warning(f"检测世界时间发生错误, 时间检测将被跳过.")
            return False
        except Exception as e:
            logger.warning(f"检测世界时间发生错误, 时间检测将被跳过.")
            show_exception(e)
            return False

    async def get_session_str_from_telethon(self, account: TelegramAccount):
        from telethon import TelegramClient
        from telethon.sessions import StringSession

        telethon_proxy = None
        if self.proxy:
            telethon_proxy = {
                "proxy_type": self.proxy.scheme,
                "addr": self.proxy.hostname,
                "port": self.proxy.port,
            }
            if self.proxy.username:
                telethon_proxy["username"] = self.proxy.username
            if self.proxy.password:
                telethon_proxy["password"] = self.proxy.password

        with tempfile.NamedTemporaryFile() as tmp_file:
            client = TelegramClient(
                tmp_file.name,
                api_id=account.api_id or API_ID,
                api_hash=account.api_hash or API_HASH,
                system_version="4.16.30-vxEMBY",
                device_model="A320MH",
                app_version=__version__,
                proxy=telethon_proxy,
            )

            msg1 = f'请输入 "{account.phone}" 的两步验证密码 (不显示, 按回车确认)'
            password_callback = lambda: Prompt.ask(" " * 23 + msg1, password=True, console=var.console)
            msg2 = f'请输入 "{account.phone}" 的登陆验证码 (按回车确认)'
            code_callback = lambda: Prompt.ask(" " * 23 + msg2, console=var.console)

            for _ in range(3):
                try:
                    await TelethonUtils(client).start(
                        phone=account.phone,
                        password=password_callback,
                        code_callback=code_callback,
                    )
                    session_string = StringSession.save(client.session)
                    me = await client.get_me()
                    user_id = me.id
                    user_bot = me.bot
                except asyncio.IncompleteReadError:
                    logger.warning(f'登录账号 "{account.phone}" 时发生网络错误, 将在 3 秒后重试.')
                    await asyncio.sleep(1)
                else:
                    break
                finally:
                    await client.disconnect()
            else:
                return None

        session = StringSession(session_string)
        Dt = Storage.SESSION_STRING_FORMAT
        return (
            base64.urlsafe_b64encode(
                struct.pack(
                    Dt,
                    session.dc_id,
                    int(account.api_id or API_ID),
                    None,
                    session.auth_key.key,
                    user_id,
                    user_bot,
                )
            )
            .decode()
            .rstrip("=")
        )

    async def login(self, account: TelegramAccount, use_telethon=True):
        try:
            self.basedir.mkdir(parents=True, exist_ok=True)
            logger.info(f'登录至账号 "{account.phone}", 请耐心等待.')
            for _ in range(3):
                session_str_src = None
                session_str = account.session
                if session_str:
                    session_str_src = "session"
                else:
                    session_str_key = f"telegram.session_str.{account.get_config_key()}"
                    session_str = cache.get(session_str_key)
                old_login_file = config.basedir / f"{account.phone}.login"
                if not session_str and old_login_file.exists():
                    try:
                        session_str = old_login_file.read_text().strip()
                        cache.set(session_str_key, session_str)
                        old_login_file.unlink()  # Delete old file after migration
                        session_str_src = "cache"
                        logger.info(f'从旧登录文件迁移账号 "{account.phone}" 的登录凭据至缓存.')
                    except Exception as e:
                        logger.warning(f"读取旧版本登录文件时发生错误, 请重新登陆.")
                if session_str:
                    session_str_src = "cache"
                if session_str:
                    logger.debug(
                        f'账号 "{account.phone}" 登录凭据存在, 仅内存模式{"启用" if self.in_memory else "禁用"}.'
                    )
                else:
                    logger.debug(
                        f'账号 "{account.phone}" 登录凭据不存在, 即将进入登录流程, 仅内存模式{"启用" if self.in_memory else "禁用"}.'
                    )
                    if use_telethon:
                        logger.debug("选择使用 Telethon 进行首次登陆, 并导出会话数据至 Pyrogram.")
                        try:
                            session_str = await self.get_session_str_from_telethon(account)
                        except EOFError:
                            logger.warning(
                                "非可交互终端, 无法输入验证码, 如果您使用 docker 请使用 docker -it 运行, 否则请使用可交互终端."
                            )
                            logger.error(f'登录账号 "{account.phone}" 时发生异常, 将被跳过.')
                            return None
                        if session_str:
                            logger.info("请耐心等待, 正在登陆.")
                            await asyncio.sleep(5)
                        else:
                            logger.warning(f'登录账号 "{account.phone}" 尝试次数超限, 将被跳过.')
                            return None
                try:
                    client = Client(
                        app_version=__version__,
                        device_model="A320MH",
                        name=account.phone,
                        system_version="4.16.30-vxEmby",
                        api_id=account.api_id or API_ID,
                        api_hash=account.api_hash or API_HASH,
                        phone_number=account.phone,
                        session_string=session_str,
                        in_memory=self.in_memory,
                        proxy=self.proxy.model_dump() if self.proxy else None,
                        workdir=str(self.basedir),
                        sleep_threshold=30,
                        workers=64,
                    )
                    try:
                        await asyncio.wait_for(client.start(), 20)
                    except asyncio.TimeoutError:
                        if self.proxy:
                            logger.error(f"无法连接到 Telegram 服务器, 请检查您代理的可用性.")
                            continue
                        else:
                            logger.error(f"无法连接到 Telegram 服务器, 请检查您的网络.")
                            continue
                    else:
                        session_str = await client.export_session_string()
                        cache.set(session_str_key, session_str)
                        logger.debug(f'登录账号 "{client.phone_number}" 成功.')
                        return client
                except OperationalError as e:
                    logger.warning(f"内部数据库错误, 正在重置, 您可能需要重新登录.")
                    show_exception(e)
                    await client.storage.delete()
                    cache.delete(session_str_key)
                    continue
                except ApiIdPublishedFlood:
                    logger.warning(f'登录账号 "{account.phone}" 时发生 API key 限制, 将被跳过.')
                    break
                except (Unauthorized, AuthKeyDuplicated) as e:
                    await client.storage.delete()
                    if session_str_src == "session":
                        logger.error(f'账号 "{account.phone}" 由于配置中提供的 session 已被注销, 将被跳过.')
                        show_exception(e)
                        return None
                    elif session_str_src == "cache":
                        logger.error(f'账号 "{account.phone}" 已被注销, 将在 3 秒后重新登录.')
                        show_exception(e)
                        cache.delete(session_str_key)
                        continue
                    else:
                        logger.error(f'账号 "{account.phone}" 已被注销, 将在 3 秒后重新登录.')
                        show_exception(e)
                    await asyncio.sleep(3)
                except KeyError as e:
                    logger.warning(
                        f'登录账号 "{account.phone}" 时发生异常, 可能是由于网络错误, 将在 3 秒后重试.'
                    )
                    show_exception(e)
                    await asyncio.sleep(3)
            else:
                logger.error(f'登录账号 "{account.phone}" 失败次数超限, 将被跳过.')
                return None
        except binascii.Error:
            logger.error(f'登录账号 "{account.phone}" 失败, 由于您在配置文件中提供的 session 无效, 将被跳过.')
        except RPCError as e:
            logger.error(f'登录账号 "{account.phone}" 失败 ({e.MESSAGE.format(value=e.value)}), 将被跳过.')
            return None
        except BadMsgNotification as e:
            if "synchronized" in str(e):
                logger.error(
                    f'登录账号 "{account.phone}" 时发生异常, 可能是因为您的系统时间与世界时间差距过大, 将被跳过.'
                )
                return None
            else:
                logger.error(f'登录账号 "{account.phone}" 时发生异常, 将被跳过.')
                show_exception(e, regular=False)
                return None
        except Exception as e:
            logger.error(f'登录账号 "{account.phone}" 时发生异常, 将被跳过.')
            show_exception(e, regular=False)
            return None

    async def loginer(self, account: TelegramAccount):
        client = await self.login(account)
        async with self.lock:
            if isinstance(client, Client) and client.me:
                self.pool[account.phone] = (client, 1)
                self.phones.append(account.phone)
                await self.done.put((account, client))
                logger.debug(f"Telegram 账号池计数增加: {account.phone} => 1")
            else:
                self.pool[account.phone] = None
                await self.done.put((account, None))

    async def __aenter__(self):
        await self.test_network()
        asyncio.create_task(self.test_time())
        for a in self.accounts:
            try:
                await self.lock.acquire()
                if a.phone not in self.pool:
                    self.pool[a.phone] = asyncio.create_task(self.loginer(a))
                else:
                    if not self.pool[a.phone]:
                        await self.done.put((a, None))
                        continue
                    if isinstance(self.pool[a.phone], asyncio.Task):
                        self.lock.release()
                        await self.pool[a.phone]
                        await self.lock.acquire()
                    client, ref = self.pool[a.phone]
                    ref += 1
                    self.pool[a.phone] = (client, ref)
                    self.phones.append(a.phone)
                    await self.done.put((a, client))
                    logger.debug(f"Telegram 账号池计数增加: {a.phone} => {ref}")
            finally:
                try:
                    self.lock.release()
                except RuntimeError:
                    pass
        return self

    async def __aiter__(self):
        for _ in range(len(self.accounts)):
            account: TelegramAccount
            client: Client
            account, client = await self.done.get()
            if client:
                yield account, client

    async def __aexit__(self, type, value, tb):
        async with self.lock:
            for phone in self.phones:
                entry = self.pool.get(phone, None)
                if entry:
                    client, ref = entry
                    ref -= 1
                    self.pool[phone] = (client, ref)
                    logger.debug(f"Telegram 账号池计数降低: {phone} => {ref}")



from .pornemby_dragon_rain import PornembyDragonRainMonitor

__ignore__ = True


class TestPornembyDragonRainMonitor(PornembyDragonRainMonitor.PornembyDragonRainClickMonitor):
    name = "Pornemby 红包雨 测试"
    chat_name = "api_group"
    chat_user = "embykeeper_test_bot"
    chat_allow_outgoing = True



from . import BotCheckin

__ignore__ = True


class TemplateBCheckin(BotCheckin):
    bot_checkin_cmd = "/checkin"
    bot_use_captcha = False


def use(**kw):
    return type("TemplatedClass", (TemplateBCheckin,), kw)



from ._templ_a import TemplateACheckin

__ignore__ = True


class StarcatCheckin(TemplateACheckin):
    name = "StarCat"
    bot_username = "StarCatBot"
    templ_panel_keywords = ["請在下方選擇您要使用的功能"]
    additional_auth = ["prime"]



import random

from embykeeper.utils import to_iterable

from . import BotCheckin


class PandaTVGroupCheckin(BotCheckin):
    name = "PandaTV 群组发言"
    chat_name = "PandaTV_Emby_Bot"
    bot_checkin_cmd = "签到"
    skip = 14

    async def send_checkin(self, **kw):
        cmd = random.choice(to_iterable(self.bot_checkin_cmd))
        await self.send(cmd)
        await self.finish(message="群组发言已发送")



from __future__ import annotations

from asyncio import Event
import asyncio
from datetime import datetime
from enum import IntEnum, auto
from typing import TYPE_CHECKING, Callable, Dict, List
import random
import string
from loguru import logger

from rich.text import Text
from pydantic import BaseModel, PrivateAttr

from .utils import to_iterable
from .cache import cache

if TYPE_CHECKING:
    from loguru import Logger

_running_runs: Dict[str, RunContext] = {}


class RunStatus(IntEnum):
    CATAGORY = auto()
    PENDING = auto()
    INITIALIZING = auto()
    RUNNING = auto()
    SUCCESS = auto()
    NONEED = auto()
    FAIL = auto()
    CANCELLED = auto()
    ERROR = auto()
    SKIP = auto()
    IGNORE = auto()
    RESCHEDULE = auto()


class LogRecord(BaseModel):
    level: str
    message: str
    time: datetime


class RunContext(BaseModel):
    _finished: Event = PrivateAttr(default_factory=Event)
    _started: Event = PrivateAttr(default_factory=Event)
    _cancel: Callable = PrivateAttr(default=None)
    _handler_id: int = PrivateAttr(default=None)

    id: str
    parent_ids: List[str] = []
    description: str = None
    status: RunStatus = RunStatus.PENDING
    status_info: str = None
    log: List[LogRecord] = []
    duration: float = None
    start_time: datetime = None
    end_time: datetime = None
    next_time: datetime = None
    reschedule: int = None

    def start(self, status: RunStatus = RunStatus.RUNNING):
        """开始任务，设置开始时间和状态"""
        self.start_time = datetime.now()
        self.set(status)
        self._started.set()

    def set(self, status: RunStatus = None):
        """设置状态"""

        if status:
            self.status = status
            self.log.append(
                LogRecord(level="DEBUG", message=f"任务状态已设置为 {status.name}", time=datetime.now())
            )

    def finish(self, status: RunStatus = None, status_info: str = None):
        """完成任务，记录状态和时间，并保存到缓存"""

        # 设置结束状态
        self.set(status)
        if status_info:
            self.status_info = status_info
        self.end_time = datetime.now()

        # 计算持续时间
        if self.start_time:
            self.duration = (self.end_time - self.start_time).total_seconds()

        # 从运行中任务列表移除
        if self.id in _running_runs:
            del _running_runs[self.id]

        # 设置完成事件
        self._finished.set()

        # 移除logger handler
        if self._handler_id is not None:
            try:
                logger.remove(self._handler_id)
            except ValueError:
                pass

        # 保存到缓存
        self.save()

        return self

    def save(self):
        """保存当前任务到缓存"""
        cache.set(f"runinfo.{self.id}", self.model_dump_json())

    @classmethod
    def cancel_all(cls):
        """取消所有运行中的任务"""
        for run in list(_running_runs.values()):
            run.cancel_tree()
            if run.status != RunStatus.CATAGORY:
                run.finish(RunStatus.CANCELLED, "任务被取消")

    def bind_logger(self, logger: Logger):
        """将 loguru logger 绑定到当前任务"""
        return logger.bind(run_id=self.id)

    @classmethod
    def prepare(cls, description: str = None, parent_ids: List[str] = None):
        """生成一个新的任务上下文"""

        # 生成随机6位ID (大写字母和数字) 的运行时
        chars = string.ascii_uppercase + string.digits
        run_id = "".join(random.choices(chars, k=6))
        run = cls(id=run_id, parent_ids=to_iterable(parent_ids))
        run.description = description

        # 设置对 loguru 的监控
        def log_sink(message):
            record = message.record
            if record["extra"].get("run_id") == run_id:
                log_record = LogRecord(
                    level=record["level"].name.upper(),
                    message=record["message"],
                    time=record["time"],
                )
                run.log.append(log_record)

        # 添加日志处理器
        run._handler_id = logger.add(log_sink, filter=lambda record: "run_id" in record["extra"])

        # 添加到运行中任务列表
        _running_runs[run_id] = run

        # 如果有父任务，记录父子关系
        if parent_ids:
            for parent_id in parent_ids:
                children = cache.get(f"runinfo.children.{parent_id}", [])
                if run_id not in children:
                    children.append(run_id)
                    cache.set(f"runinfo.children.{parent_id}", children)

        return run

    @classmethod
    def get(cls, run_id: str) -> "RunContext":
        # 优先从运行中任务获取
        if run_id in _running_runs:
            return _running_runs[run_id]

        # 从缓存加载
        run_json = cache.get(f"runinfo.{run_id}")
        if run_json:
            return cls.model_validate_json(run_json)
        return None

    def get_parents(self):
        """获取所有父任务"""
        parents = []
        for parent_id in self.parent_ids:
            parent = RunContext.get(parent_id)
            if parent:
                parents.append(parent)
        return parents

    def get_children(self):
        """获取所有子任务"""
        children = []
        child_ids = cache.get(f"runinfo.children.{self.id}", [])
        for child_id in child_ids:
            child = RunContext.get(child_id)
            if child:
                children.append(child)
        return children

    def yield_logs(self, reverse: bool = False, include_children: bool = False):
        """按时间顺序产出日志记录"""
        logs = self.log.copy()

        if include_children:
            for child in self.get_children():
                logs.extend(child.log)

        # 确保所有日志都有时间戳
        for log in logs:
            if log.time is None:
                log.time = datetime.now()

        # 按时间排序
        logs.sort(key=lambda x: x.time, reverse=reverse)
        yield from logs

    def log_sink(self, message):
        record = message.record
        if record["extra"].get("run_id") == self.id:
            log_record = LogRecord(
                level=record["level"].name.upper(),
                message=Text(record["message"]).plain,
                time=record["time"],
            )
            self.log.append(log_record)

    @classmethod
    def run(cls, func: Callable, description: str = None, parent_ids: List[str] = None):
        async def runner():
            ctx = RunContext.prepare(
                description=description or func.__name__,
                parent_ids=parent_ids,
            )
            task = asyncio.create_task(func(ctx))
            ctx._cancel = task.cancel
            try:
                return await task
            except asyncio.CancelledError:
                ctx.finish(RunStatus.CANCELLED, "任务被取消")
                raise
            except Exception as e:
                ctx.finish(RunStatus.ERROR, f"任务发生错误")
                raise

        return runner()

    def get_running_children(self):
        """获取所有正在运行的子任务"""
        children = []
        child_ids = cache.get(f"runinfo.children.{self.id}", [])
        for child_id in child_ids:
            if child_id in _running_runs:
                children.append(_running_runs[child_id])
        return children

    def cancel_tree(self):
        """取消当前任务及其所有运行中的子任务"""
        # 先取消所有子任务
        for child in self.get_running_children():
            if child._cancel:
                child._cancel()

        # 取消自身任务
        if self._cancel:
            self._cancel()

    @classmethod
    def get_or_create(
        cls,
        run_id: str = None,
        description: str = None,
        parent_ids: List[str] = None,
        status: RunStatus = RunStatus.CATAGORY,
    ):
        """获取现有任务或创建新任务"""

        if run_id:
            existing = cls.get(run_id)
            if existing:
                return existing
        ctx = cls.prepare(description=description, parent_ids=parent_ids)
        if status:
            ctx.set(status)
        return ctx



from datetime import time

from .smart_pornemby import SmartPornembyMessager

__ignore__ = True


class TestSmartPornembyMessager(SmartPornembyMessager):
    name = "Pornemby 智能水群测试"
    chat_name = "api_group"
    msg_per_day = 1000
    min_msg_gap = 0
    at = (time(0, 0), time(23, 59))



import asyncio
from datetime import datetime
from ._smart import SmartMessager
from ..lock import (
    pornemby_nohp,
    pornemby_messager_enabled,
    pornemby_messager_mids,
    pornemby_alert,
)

__ignore__ = True


class SmartPornembyMessager(SmartMessager):
    name = "Pornemby"
    chat_name = "pornemby"
    default_messages = "pornemby-common-wl@latest.yaml"
    additional_auth = ["pornemby_pack"]
    msg_per_day = 100

    async def init(self):
        self.lock = asyncio.Lock()
        pornemby_messager_enabled[self.me.id] = True
        pornemby_messager_mids[self.me.id] = []
        return True

    async def send(self, dummy=False):
        if pornemby_alert.get(self.me.id, False):
            self.log.info(f"由于风险急停取消发送.")
            return
        nohp_date = pornemby_nohp.get(self.me.id, None)
        if nohp_date and nohp_date >= datetime.today().date():
            self.log.info(f"取消发送: 血量已耗尽.")
            return
        msg = await super().send(dummy=dummy)
        if msg:
            pornemby_messager_mids[self.me.id].append(msg.id)
        return msg



from pyrogram.types import Message
from pyrogram.errors import RPCError
from thefuzz import fuzz

from . import AnswerBotCheckin

__ignore__ = True


class LJYYCheckin(AnswerBotCheckin):
    ocr = "uchars4@v1"

    name = "垃圾影音"
    bot_username = "zckllflbot"
    bot_captcha_len = 4
    bot_use_history = 20
    bot_text_ignore = "下列选项"

    async def retry(self):
        if self.message:
            try:
                await self.message.click()
            except (RPCError, TimeoutError):
                pass
        await super().retry()

    async def on_captcha(self, message: Message, captcha: str):
        async with self.operable:
            if not self.message:
                await self.operable.wait()
            match = [(k, fuzz.ratio(k, captcha)) for k in self.get_keys(self.message)]
            max_k, max_r = max(match, key=lambda x: x[1])
            await self.message.click(max_k)



from __future__ import annotations

import asyncio
import random
from typing import List
from loguru import logger

from embykeeper.config import config
from embykeeper.runinfo import RunContext, RunStatus
from embykeeper.schedule import Scheduler
from embykeeper.schema import SubsonicAccount
from embykeeper.utils import show_exception

from .player import SubsonicPlayer

logger = logger.bind(scheme="subsonic")


class SubsonicManager:
    def get_spec(self, a: SubsonicAccount):
        return f"{a.username}@{a.name or a.url.host}"

    async def _listen_main(self, accounts: List[SubsonicAccount], instant: bool = False):
        if not accounts:
            return None
        logger.info("开始执行 Subsonic 保活.")
        tasks = []
        sem = asyncio.Semaphore(config.subsonic.concurrency or 100000)

        ctx = RunContext.prepare(description="使用全局设置的 Subsonic 统一保活")
        ctx.start(RunStatus.INITIALIZING)

        async def watch_wrapper(account: SubsonicAccount, sem):
            async with sem:
                try:
                    player = SubsonicPlayer(account)
                except Exception as e:
                    logger.error(f"初始化失败: {e}")
                    show_exception(e, regular=False)
                    return account, False
                if not instant:
                    wait = random.uniform(180, 360)
                    player.log.info(f"播放音频前随机等待 {wait:.0f} 秒.")
                    await asyncio.sleep(wait)
                try:
                    subsonic = await player.login()
                    if not subsonic:
                        return account, False
                    await asyncio.sleep(random.uniform(2, 5))
                    return account, await player.play(subsonic)
                except Exception as e:
                    player.log.error(f"播放任务执行失败: {e}")
                    show_exception(e, regular=False)
                    return account, False

        for account in accounts:
            if account.enabled:
                tasks.append(watch_wrapper(account, sem))

        failed_accounts = []
        successful_accounts = []
        results = await asyncio.gather(*tasks)
        for a, success in results:
            if success:
                successful_accounts.append(self.get_spec(a))
            else:
                failed_accounts.append(self.get_spec(a))
        fails = len(failed_accounts)

        if fails:
            if len(accounts) == 1:
                logger.error(f"保活失败: {', '.join(failed_accounts)}")
            else:
                logger.error(f"保活失败 ({fails}/{len(tasks)}): {', '.join(failed_accounts)}")
            return ctx.finish(RunStatus.FAIL, f"保活失败")
        if len(accounts) == 1:
            logger.bind(log=True).info(f"保活成功: {', '.join(successful_accounts)}.")
        else:
            logger.bind(log=True).info(
                f"保活成功 ({len(tasks)}/{len(tasks)}): {', '.join(successful_accounts)}."
            )
        return ctx.finish(RunStatus.SUCCESS, f"保活成功")

    async def run_all(self, instant: bool = False):
        return await self._listen_main(config.subsonic.account, instant)

    async def schedule_all(self, instant: bool = False):
        unified_accounts: List[SubsonicAccount] = []
        independent_accounts: List[SubsonicAccount] = []
        tasks = []

        # Separate accounts into global and site-specific
        for account in config.subsonic.account:
            if not account.enabled:
                continue
            if account.time_range or account.interval_days:
                independent_accounts.append(account)
            else:
                unified_accounts.append(account)

        # Schedule global accounts together
        if unified_accounts:
            on_next_time = lambda t: logger.bind(log=True).info(
                f"下一次 Subsonic 保活将在 {t.strftime('%m-%d %H:%M %p')} 进行."
            )
            scheduler = Scheduler.from_str(
                func=lambda ctx: self._listen_main(unified_accounts, instant),
                interval_days=config.emby.interval_days,
                time_range=config.emby.time_range,
                on_next_time=on_next_time,
                sid="subsonic.watch.global",
                description="Subsonic 保活任务",
            )
            tasks.append(scheduler.schedule())

        # Schedule individual site accounts
        for account in independent_accounts:
            account_spec = self.get_spec(account)
            time_range = account.time_range or config.emby.time_range
            interval = account.interval_days or config.emby.interval_days

            on_next_time = lambda t: logger.bind(log=True).info(
                f"下一次 Subsonic 账号 ({account_spec}) 的签到将在 {t.strftime('%m-%d %H:%M %p')} 进行."
            )
            scheduler = Scheduler.from_str(
                func=lambda ctx: self._watch_main([account], False),
                interval_days=interval,
                time_range=time_range,
                on_next_time=on_next_time,
                sid=f"subsonic.watch.{account_spec}",
                description=f"Subsonic 保活任务 - {account_spec}",
            )
            tasks.append(scheduler.schedule())

        if not tasks:
            logger.info("没有需要执行的 Subsonic 保活任务")
            return None

        await asyncio.gather(*tasks)



import asyncio
from json import JSONDecodeError
from urllib.parse import parse_qs, urlparse
from datetime import datetime, timezone

from curl_cffi.requests import AsyncSession, RequestsError
from pyrogram.raw.functions.messages import RequestWebView
from pyrogram.raw.functions.users import GetFullUser

from embykeeper.runinfo import RunStatus
from embykeeper.utils import format_timedelta_human, get_proxy_str, show_exception
from embykeeper.config import config

from ..link import Link
from . import BotCheckin


class NebulaCheckin(BotCheckin):
    name = "Nebula"
    bot_username = "Nebula_Account_bot"
    max_retries = 1

    async def send_checkin(self, **kw):
        bot_peer = await self.client.resolve_peer(self.bot_username)
        user_full = await self.client.invoke(GetFullUser(id=bot_peer))
        url = user_full.full_user.bot_info.menu_button.url
        url_auth = (
            await self.client.invoke(RequestWebView(peer=bot_peer, bot=bot_peer, platform="ios", url=url))
        ).url
        scheme = urlparse(url_auth)
        params = parse_qs(scheme.fragment)
        webapp_data = params.get("tgWebAppData", [""])[0]

        parsed_url = urlparse(url_auth)
        base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
        url_info = f"{base_url}/api/v1/tg/info"
        url_checkin = f"{base_url}/api/v1/tg/checkin"

        headers = {"X-Initdata": webapp_data}

        try:
            async with AsyncSession(
                proxy=get_proxy_str(config.proxy), headers=headers, impersonate="edge", allow_redirects=True
            ) as session:
                # 先获取用户信息
                resp_info = await session.get(url_info)
                info_results = resp_info.json()

                if info_results.get("message") != "Success":
                    self.log.info("签到失败: 账户错误.")
                    return await self.fail(message="账户错误")

                # 获取当前余额和下次签到时间
                current_balance = info_results["data"]["balance"]
                next_checkin_time = datetime.fromisoformat(
                    info_results["data"]["next_check_in"].replace("Z", "+00:00").split(".")[0]
                    + "."
                    + info_results["data"]["next_check_in"].split(".")[1][:6]
                    + "+00:00"
                )

                # 检查是否可以签到
                if next_checkin_time > datetime.now(timezone.utc):
                    # 获取今天24点的时间
                    today_end = datetime.now(timezone.utc).replace(
                        hour=23, minute=59, second=59, microsecond=999999
                    )
                    if next_checkin_time <= today_end:
                        # 今天还可以签到，等待到指定时间
                        sleep = next_checkin_time - datetime.now(timezone.utc)
                        self.log.info(f"即将在 {format_timedelta_human(sleep)} 后重试.")
                        self.ctx.next_time = next_checkin_time
                        return await self.finish(RunStatus.RESCHEDULE, "等待重新尝试签到")
                    else:
                        # 今天已经不能签到了
                        self.log.info("今日已经签到过了.")
                        return await self.finish(RunStatus.NONEED, "今日已签到")

                # 执行签到
                resp = await session.post(url_checkin)
                results = resp.json()
                message = results["message"]
                if any(s in message for s in ("未找到用户", "权限错误")):
                    self.log.info("签到失败: 账户错误.")
                    return await self.fail(message="账户错误")
                if "Failed" in message:
                    self.log.info("签到失败.")
                    return await self.retry()
                elif "Success" in message:
                    self.log.info(
                        f"[yellow]签到成功[/]: + {results['data']['coin']} 分 -> {current_balance + results['data']['coin']} 分."
                    )
                    return await self.finish(RunStatus.SUCCESS, "签到成功")
                else:
                    self.log.warning(f"接收到异常返回信息: {results}")
                    return await self.retry()
        except (RequestsError, OSError, JSONDecodeError) as e:
            self.log.info(f"签到失败: 无法连接签到页面 ({e.__class__.__name__}).")
            show_exception(e)
            return await self.retry()



from pyrogram.types import Message
from . import Monitor

__ignore__ = True


class TestGroupMonitor(Monitor):
    name = "多组 测试"
    chat_name = "api_group"
    chat_allow_outgoing = True
    chat_keyword = r"多组\s*(\w+)\s+(\w+)"
    chat_delay = 1

    def chat_reply(self, message: Message, keys):
        return f'接收到: "{keys[0]}" => "{keys[1]}"'



from . import BotCheckin

__ignore__ = True


class RednoseSGKCheckin(BotCheckin):
    name = "红鼻子社工库"
    bot_username = "freesgk123_bot"
    bot_checkin_cmd = "/sign"
    bot_checked_keywords = "请勿重复签到"



from . import BotCheckin

__ignore__ = True


class IngeekSGKCheckin(BotCheckin):
    name = "Ingeek 社工库"
    bot_username = "ingeeksgkbot"
    bot_checkin_cmd = "/checkin"
    bot_checked_keywords = ["今日已签到"]



import asyncio
import time
from typing import Iterable, Union

import aiofiles
import httpx
from cachetools import TTLCache
from loguru import logger

from .utils import format_byte_human, nonblocking, show_exception, to_iterable, get_proxy_str
from .config import config

logger = logger.bind(scheme="datamanager")

cdn_urls = [
    "https://raw.githubusercontent.com/emby-keeper/emby-keeper-data/main",
    "https://raw.gitmirror.com/emby-keeper/emby-keeper-data/main",
    "https://cdn.jsdelivr.net/gh/emby-keeper/emby-keeper-data",
]

versions = TTLCache(maxsize=128, ttl=600)
lock = asyncio.Lock()


async def refresh_version():
    async with nonblocking(lock):
        for data_url in cdn_urls:
            url = f"{data_url}/version"
            async with httpx.AsyncClient(
                http2=True, proxy=get_proxy_str(config.proxy), follow_redirects=True
            ) as client:
                try:
                    resp = await client.get(url)
                    if resp.status_code == 200:
                        result = resp.text
                        for l in result.splitlines():
                            if l:
                                a, b = l.split("=")
                                versions[a.strip()] = b.strip()
                        break
                    else:
                        logger.warning(f"资源文件版本信息获取失败 ({resp.status_code})")
                        return False
                except httpx.HTTPError as e:
                    continue
                except Exception as e:
                    logger.warning(f"资源文件版本信息获取失败 ({e})")
                    show_exception(e)
                    return False
        else:
            logger.warning(f"资源文件版本信息获取失败.")
            return False


async def get_datas(names: Union[Iterable[str], str], caller: str = None):
    """
    获取额外数据.
    参数:
        names: 要下载的路径列表
        caller: 请求下载的模块名, 用于消息提示
    """

    basedir = config.basedir
    existing = {}
    not_existing = []
    for name in to_iterable(names):
        if (basedir / name).is_file():
            logger.debug(f'检测到请求的本地文件: "{name}".')
            existing[name] = basedir / name
        else:
            not_existing.append(name)

    if not_existing:
        logger.info(f"{caller or '该功能'} 正在下载或更新资源文件: {', '.join(not_existing)}")

    for name in to_iterable(names):
        version_matching = False
        while True:
            if (basedir / name).is_file():
                yield basedir / name
            else:
                try:
                    for data_url in cdn_urls:
                        url = f"{data_url}/data/{name}"
                        logger.debug(f"正在尝试 URL: {url}")
                        async with httpx.AsyncClient(
                            http2=True, proxy=get_proxy_str(config.proxy), follow_redirects=True
                        ) as client:
                            try:
                                resp = await client.get(url)
                                if resp.status_code == 200:
                                    file_size = int(resp.headers.get("content-length", 0))
                                    logger.info(f"开始下载: {name} ({format_byte_human(file_size)})")
                                    async with aiofiles.open(basedir / name, mode="wb+") as f:
                                        timer = time.time()
                                        length = 0
                                        async for chunk in resp.aiter_bytes(chunk_size=512):
                                            if time.time() - timer > 3:
                                                timer = time.time()
                                                logger.info(
                                                    f"正在下载: {name} ({format_byte_human(length)} / {format_byte_human(file_size)})"
                                                )
                                            await f.write(chunk)
                                            length += len(chunk)
                                    logger.info(f"下载完成: {name} ({format_byte_human(file_size)})")
                                    yield basedir / name
                                    break
                                elif resp.status_code in (403, 404) and not version_matching:
                                    await refresh_version()
                                    if name in versions:
                                        logger.debug(f'解析版本 "{name}" -> "{versions[name]}"')
                                        name = versions[name]
                                        version_matching = True
                                        break
                                    else:
                                        logger.warning(f"下载失败: {name} ({resp.status_code})")
                                        yield None
                                        break
                                else:
                                    logger.warning(f"下载失败: {name} ({resp.status_code})")
                                    yield None
                                    break
                            except httpx.HTTPError as e:
                                (basedir / name).unlink(missing_ok=True)
                                continue
                            except Exception as e:
                                (basedir / name).unlink(missing_ok=True)
                                logger.warning(f"下载失败: {name} ({e})")
                                show_exception(e)
                                yield None
                                break
                    else:
                        logger.warning(f"下载失败: {name}.")
                        yield None
                        continue
                except KeyboardInterrupt:
                    (basedir / name).unlink(missing_ok=True)
                    raise
            if not version_matching:
                break


async def get_data(name: str, caller: str = None):
    async for data in get_datas(name, caller):
        return data



import asyncio
from io import BytesIO
import random
import re

import httpx
from pyrogram.types import Message
from pyrogram.errors import MessageIdInvalid
from PIL import Image
import numpy as np

from embykeeper.config import config
from embykeeper.utils import show_exception, get_proxy_str

from ..lock import pornemby_alert
from . import Monitor


class _PornembyExamResultMonitor(Monitor):
    name = "Pornemby 科举答案"
    chat_keyword = r"问题\d*：(.*?)\n+答案为：([ABCD])\n+([A-Z-\d]+)"
    additional_auth = ["pornemby_pack"]
    allow_edit = True

    async def on_trigger(self, message: Message, key, reply):
        self.log.info(f"本题正确答案为 {key[1]} ({key[2]}).")


class _PornembyExamAnswerMonitor(Monitor):
    name = "Pornemby 科举"
    chat_user = [
        "pornemby_question_bot",
        "PronembyTGBot2_bot",
        "PronembyTGBot3_bot",
        "PornembyBot",
        "Porn_Emby_Bot",
        "Porn_Emby_Scriptbot",
    ]
    chat_keyword = (
        r"问题\d*：根据以上封面图，猜猜是什么番号？\n+A:(.*)\n+B:(.*)\n+C:(.*)\n+D:(.*)\n(?!\n*答案)"
    )
    additional_auth = ["pornemby_pack"]
    allow_edit = True

    key_map = {
        "A": ["A", "🅰"],
        "B": ["B", "🅱"],
        "C": ["C", "🅲"],
        "D": ["D", "🅳"],
    }

    async def get_cover_image_javdatabase(self, code: str):
        # 添加重试次数
        max_retries = 3
        retry_count = 0

        while retry_count < max_retries:
            try:
                proxy = get_proxy_str(config.proxy)
                # 使用 httpx 创建异步客户端
                async with httpx.AsyncClient(
                    http2=True,
                    proxy=proxy,
                    verify=False,
                    follow_redirects=True,
                ) as client:
                    detail_url = f"https://www.javdatabase.com/movies/{code.lower()}/"
                    response = await client.get(detail_url)
                    if response.status_code != 200:
                        self.log.warning(
                            f"获取影片详情失败: 网址访问错误: {detail_url} ({response.status_code})."
                        )
                        retry_count += 1
                        if retry_count < max_retries:
                            self.log.info(f"正在进行第 {retry_count + 1} 次重试...")
                            continue
                        return None
                    html = response.content.decode()
                    pattern = (
                        f'<div id="thumbnailContainer".*(https://www.javdatabase.com/covers/thumb/.*/.*.webp)'
                    )
                    match = re.search(pattern, html)
                    if not match:
                        self.log.warning(
                            f"获取封面图片失败: 未找到图片: {detail_url} ({response.status_code})."
                        )
                        return None
                    img_url = match.group(1)
                    # 下载封面图片
                    img_response = await client.get(img_url)
                    if img_response.status_code == 200:
                        return BytesIO(img_response.content)
                    else:
                        self.log.warning(
                            f"获取封面图片失败: 网址访问错误: {img_url} ({img_response.status_code})."
                        )
                        return None

            except Exception as e:
                retry_count += 1
                if retry_count < max_retries:
                    self.log.info(
                        f"获取封面图片失败，正在进行第 {retry_count + 1} 次重试: {e.__class__.__name__}: {str(e)}"
                    )
                    continue
                self.log.warning(f"获取封面图片失败: {e.__class__.__name__}: {str(e)}")
                show_exception(e)
                return None

            # 如果执行到这里说明成功获取了图片，直接返回
            break

        return None

    async def get_cover_image_r18_dev(self, code: str):
        # 添加重试次数
        max_retries = 3
        retry_count = 0

        while retry_count < max_retries:
            try:
                proxy = get_proxy_str(config.proxy)
                # 使用 httpx 创建异步客户端
                async with httpx.AsyncClient(
                    http2=True,
                    proxy=proxy,
                    verify=False,
                    follow_redirects=True,
                ) as client:
                    # 先获取 content_id
                    detail_url = f"https://r18.dev/videos/vod/movies/detail/-/dvd_id={code.lower()}/json"
                    # 获取 content_id
                    response = await client.get(detail_url)
                    if response.status_code != 200:
                        self.log.warning(
                            f"获取影片详情失败: 网址访问错误: {detail_url} ({response.status_code})."
                        )
                        retry_count += 1
                        if retry_count < max_retries:
                            self.log.info(f"正在进行第 {retry_count + 1} 次重试...")
                            continue
                        return None
                    detail_json = response.json()
                    content_id = detail_json.get("content_id")
                    if not content_id:
                        self.log.warning(f"获取影片详情失败: 无法获取 content_id: {detail_url}")
                        return None

                    # 获取封面图片 URL
                    combined_url = f"https://r18.dev/videos/vod/movies/detail/-/combined={content_id}/json"
                    response = await client.get(combined_url)
                    if response.status_code != 200:
                        self.log.warning(
                            f"获取封面详情失败: 网址访问错误: {combined_url} ({response.status_code})."
                        )
                        return None
                    combined_json = response.json()
                    jacket_url = combined_json.get("jacket_thumb_url")
                    if not jacket_url:
                        self.log.warning(f"获取封面详情失败: 无法获取封面URL: {combined_url}")
                        return None

                    # 下载封面图片
                    img_response = await client.get(jacket_url)
                    if img_response.status_code == 200:
                        return BytesIO(img_response.content)
                    else:
                        self.log.warning(
                            f"获取封面图片失败: 网址访问错误: {jacket_url} ({img_response.status_code})."
                        )
                        return None

            except Exception as e:
                retry_count += 1
                if retry_count < max_retries:
                    self.log.info(
                        f"获取封面图片失败，正在进行第 {retry_count + 1} 次重试: {e.__class__.__name__}: {str(e)}"
                    )
                    continue
                self.log.warning(f"获取封面图片失败: {e.__class__.__name__}: {str(e)}")
                show_exception(e)
                return None

            # 如果执行到这里说明成功获取了图片，直接返回
            break

        return None

    def compare_images(self, img1_bytes: BytesIO, img2_bytes: BytesIO) -> float:
        try:
            img1 = Image.open(img1_bytes).convert("RGB").resize((100, 100))
            img2 = Image.open(img2_bytes).convert("RGB").resize((100, 100))

            arr1 = np.array(img1)
            arr2 = np.array(img2)
            mse = np.mean((arr1 - arr2) ** 2)

            similarity = 1 / (1 + mse)
            return similarity
        except Exception as e:
            self.log.debug(f"图片比较失败: {e}")
            return 0

    async def on_trigger(self, message: Message, key, reply):
        if not message.photo or not message.reply_markup:
            return
        if pornemby_alert.get(self.client.me.id, False):
            self.log.info(f"由于风险急停不作答.")
            return
        if random.random() > self.config.get("possibility", 1.0):
            self.log.info(f"由于概率设置不作答.")
            return

        question_photo = await message.download(in_memory=True)

        codes = [re.sub(r"-\w$", "", k) for k in key]

        async def get_cover_with_timeout(code):
            try:
                return code, await asyncio.wait_for(self.get_cover_image_javdatabase(code), timeout=10)
            except asyncio.TimeoutError:
                self.log.debug(f"获取 {code} 封面超时")
                return code, None

        cover_tasks = [get_cover_with_timeout(code) for code in codes]
        covers = await asyncio.gather(*cover_tasks)
        max_similarity = -1
        best_code = None
        for code, cover in covers:
            if cover is None:
                continue
            question_photo.seek(0)
            cover.seek(0)
            similarity = self.compare_images(question_photo, cover)
            self.log.debug(f"番号 {code} 相似度: {similarity:.4f}")
            if similarity > max_similarity:
                max_similarity = similarity
                best_code = code
        if best_code:
            result = ["A", "B", "C", "D"][codes.index(best_code)]
            self.log.info(f"选择相似度最高的番号: {best_code} ({result}) (相似度: {max_similarity:.4f})")
            buttons = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            answer_options = self.key_map[result]
            for button_text in buttons:
                if any((o in button_text) for o in answer_options):
                    try:
                        await message.click(button_text)
                    except (TimeoutError, MessageIdInvalid):
                        pass
                    break
            else:
                self.log.info(f"点击失败: 未找到匹配的按钮文本 {result}.")
        else:
            self.log.warning("未找到匹配的封面图片")


class PornembyExamMonitor:
    class PornembyExamResultMonitor(_PornembyExamResultMonitor):
        chat_name = "Pornemby"

    class PornembyExamAnswerMonitor(_PornembyExamAnswerMonitor):
        chat_name = "Pornemby"



import httpx
from pyrogram.types import Message
from pyrogram.raw.functions.messages import AcceptUrlAuth
from pyrogram.raw.types import UrlAuthResultAccepted
from faker import Faker

from embykeeper.utils import get_proxy_str
from embykeeper.config import config

from . import BotCheckin

__ignore__ = True


class JMSCheckin(BotCheckin):
    name = "卷毛鼠"
    bot_username = "jmsembybot"
    bot_checked_keywords = "请明天再来签到"

    async def message_handler(self, client, message: Message):
        if message.reply_markup:
            keys = [k for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "点我签到" in k.text:
                    r: UrlAuthResultAccepted = await self.client.invoke(
                        AcceptUrlAuth(
                            peer=await self.client.resolve_peer(message.chat.id),
                            msg_id=message.id,
                            button_id=k.login_url.button_id,
                            url=k.login_url.url,
                        )
                    )
                    url = r.url
                    for _ in range(1, 3):
                        async with httpx.AsyncClient(
                            http2=True, proxies=get_proxy_str(config.proxy)
                        ) as client:
                            resp = await client.get(url, headers={"User-Agent": Faker().safari()})
                            if resp.status_code == 200:
                                return
        return await super().message_handler(client, message)



import asyncio
import platform
import subprocess
import httpx
from pathlib import Path
import stat
from typing import Optional

from .schema import ProxyConfig
from .utils import get_proxy_str
from .config import config


class WSSocks:
    BASE_URL = "https://github.com/zetxtech/wssocks/releases/download/v1.4.2"

    PLATFORM_MAPPING = {
        "Linux": {
            "x86_64": "wssocks-linux-amd64",
            "aarch64": "wssocks-linux-arm64",
        },
        "Darwin": {
            "x86_64": "wssocks-darwin-amd64",
            "arm64": "wssocks-darwin-arm64",
        },
        "Windows": {
            "AMD64": "wssocks-windows-amd64.exe",
        },
    }

    def __init__(self, proxy=None):
        """Initialize WSSocks handler

        Args:
            cache_dir: Directory to store downloaded files. Defaults to ~/.cache/embykeeper
        """
        self.system = platform.system()
        self.machine = platform.machine()
        self.process: Optional[subprocess.Popen] = None
        self._proxy = proxy

    @property
    def proxy_str(self):
        return get_proxy_str(self._proxy or config.proxy)

    @property
    def executable_path(self) -> Path:
        """Get path to the executable"""
        exe_name = "wssocks.exe" if self.system == "Windows" else "wssocks"
        return config.basedir / exe_name

    def get_download_url(self) -> str:
        """Generate download URL based on current platform"""
        try:
            filename = self.PLATFORM_MAPPING[self.system][self.machine]
            return f"{self.BASE_URL}/{filename}"
        except KeyError:
            raise RuntimeError(f"Unsupported platform: {self.system} {self.machine}")

    async def download(self) -> None:
        """Download wssocks binary asynchronously"""

        url = self.get_download_url()
        temp_path = config.basedir / self.PLATFORM_MAPPING[self.system][self.machine]

        # Download file to temporary path using httpx
        async with httpx.AsyncClient(proxy=self.proxy_str, http2=True, follow_redirects=True) as client:
            response = await client.get(url)
            response.raise_for_status()

            # Write response content to file
            with open(temp_path, "wb") as f:
                f.write(response.content)

        # Rename to standard executable name
        temp_path.rename(self.executable_path)

        # Set executable permission on Unix
        if self.system != "Windows":
            self.executable_path.chmod(self.executable_path.stat().st_mode | stat.S_IEXEC)

    async def ensure_binary(self) -> None:
        """Ensure binary exists and download if necessary"""
        if not self.executable_path.exists():
            await self.download()

    async def execute(self, *args) -> subprocess.Popen:
        """Execute wssocks with given arguments and return Popen object"""
        await self.ensure_binary()
        return subprocess.Popen(
            [str(self.executable_path), *args],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

    async def start(self, host: str, token: str, connector_token: str, proxy: ProxyConfig = None) -> bool:
        """Start wssocks client server

        Args:
            host: Server address with port
            token: Authentication token
            connector_token: Connector token
            proxy_dict: Proxy configuration dictionary containing scheme, hostname, port, and optional username/password
        """
        if self.process and self.process.poll() is None:
            raise RuntimeError("WSSocks is already running")

        args = ["client", "-u", host, "-r", "-t", token, "-T", "1", "-c", connector_token, "-d", "-E"]
        proxy_str = get_proxy_str(proxy)
        if proxy_str:
            args.extend(["-x", proxy_str])

        self.process = await self.execute(*args)

        await asyncio.sleep(3)
        if self.process.poll() is not None:
            stdout, stderr = self.process.communicate()
            return stderr.decode()
        return None

    def stop(self) -> None:
        """Stop wssocks server if running"""
        if self.process:
            if self.process.poll() is None:  # Process is still running
                self.process.terminate()  # Try graceful shutdown first
                try:
                    self.process.wait(timeout=5)  # Wait up to 5 seconds
                except subprocess.TimeoutExpired:
                    self.process.kill()  # Force kill if not terminated
            self.process = None



import asyncio
from datetime import datetime

from pyrogram.types import Message
from pyrogram.enums import MessageEntityType

from ..lock import pornemby_nohp, pornemby_messager_enabled

from . import Monitor


class PornembyNoHPMonitor(Monitor):
    name = "Pornemby 血量耗尽停止发言"
    chat_user = ["PronembyTGBot2_bot", "PronembyTGBot3_bot", "PornembyBot", "Porn_Emby_Bot"]
    chat_name = "Pornemby"
    chat_keyword = "(.*)血量已耗尽。"
    additional_auth = ["pornemby_pack"]
    allow_edit = True

    async def on_trigger(self, message: Message, key, reply):
        for me in message.entities:
            if me.type == MessageEntityType.TEXT_MENTION:
                if me.user.id == self.client.me.id:
                    pornemby_nohp[self.client.me.id] = datetime.today().date()
                    self.log.info("检测到 Pornemby 血量耗尽, 已停止今日水群.")

    async def init(self):
        interval = 1
        while True:
            if pornemby_messager_enabled.get(self.client.me.id, False):
                return True
            await asyncio.sleep(interval)
            interval += 1



from pyrogram.types import Message

from embykeeper.var import console

from ..link import Link
from . import Monitor

__ignore__ = True


class TerminusExamMonitor(Monitor):
    name = "终点站考试辅助"
    chat_name = "EmbyPublicBot"
    chat_keyword = r"(.*?(?:\n(?!本题贡献者).*?)*)\s+本题贡献者"
    allow_edit = True
    additional_auth = ["gpt"]
    debug_no_log = True
    trigger_interval = 0
    trigger_sem = None

    async def init(self):
        self.log.info("您已开启终点站考试辅助, 正在测试答题服务状态.")
        result, _ = await Link(self.client).terminus_answer("请输出'正常'两个字!")
        if result:
            self.log.info("终点站考试辅助正常已启用, 请在机器人触发考试开始.")
            return True
        else:
            self.log.warning("终点站考试辅助服务状态不正常, 请稍后再试.")
            return False

    async def on_trigger(self, message: Message, key, reply):
        self.log.info(f"新题: {key}, 解析中...")
        if message.reply_markup and message.reply_markup.inline_keyboard:
            options = []
            for row in message.reply_markup.inline_keyboard:
                for button in row:
                    options.append(button.text)
        question = f"问题:\n{key}\n选项:\n" + "\n".join(f"- {option}" for option in options)
        result, by = await Link(self.client).terminus_answer(question)
        if result:
            console.rule(title=f"{by} 给出答案")
            console.print(result)
            console.rule()
        else:
            self.log.warning("解析失败! 请自行回答.")



from .pornemby_alert import PornembyAlertMonitor

__ignore__ = True


class TestPornembyAlertMonitor(PornembyAlertMonitor):
    name = "Pornemby 风险急停 测试"
    chat_name = "api_group"
    chat_allow_outgoing = True



import random

from embykeeper.utils import to_iterable

from . import BotCheckin

__ignore__ = True


class MarmotGroupCheckin(BotCheckin):
    name = "Marmot 群组发言"
    chat_name = -1001975531465
    bot_checkin_cmd = ["签到", "打劫", "没币了", "低保", "打卡", "冒泡"]
    skip = 14

    async def send_checkin(self, **kw):
        cmd = random.choice(to_iterable(self.bot_checkin_cmd))
        await self.send(cmd)
        await self.finish(message="群组发言已发送")



import base64
import os
from typing import List

from loguru import logger
from rich.prompt import Confirm, Prompt, IntPrompt, Confirm

from .config import ConfigManager, config
from .schema import Config, TelegramAccount, EmbyAccount
from .var import console
from .log import pad
from . import __url__


async def convert_session(accounts: List[TelegramAccount]):
    from .telegram.session import ClientsSession

    for a in accounts:
        success = False
        async with ClientsSession([a]) as clients:
            async for a, tg in clients:
                a.session = await tg.export_session_string()
                logger.info(f'账号 "{a.phone}" 登陆成功.')
                success = True
            if not success:
                logger.error(f'账号 "{a.phone}" 登陆失败, 已被禁用.')
                a.enabled = False
    return accounts


async def interactive_config():
    from tomlkit import item

    cfg = Config()
    config.set(cfg)
    pad = " " * 23
    logger.info("我们将为您生成配置, 需要您根据提示填入信息, 并按回车确认.")
    logger.info(f"配置帮助详见: {__url__}.")
    logger.info(f"若需要重新开始, 请点击右上方的刷新按钮.")
    logger.info(f"若您需要更加高级的配置, 请使用右上角的 Config 按钮以修改配置文件.")

    mongodb_url = Prompt.ask(
        pad + "请输入 MongoDB 连接地址 [dark_green](mongodb://user:pass@host:port)[/]", console=console
    )
    cfg.mongodb = mongodb_url

    telegram_accounts = cfg.telegram.account
    while True:
        if len(telegram_accounts) > 0:
            logger.info(
                f"您当前填写了 {len(telegram_accounts)} 个 Telegram 账号信息: "
                f'{", ".join([t.phone for t in telegram_accounts])}'
            )
            more = Confirm.ask(pad + "是否继续添加?", default=False, console=console)
        else:
            more = Confirm.ask(pad + "是否添加 Telegram 账号?", default=True, console=console)
        if not more:
            break
        phone = Prompt.ask(
            pad + "请输入您的 Telegram 账号 (带国家区号) [dark_green](+861xxxxxxxxxx)[/]",
            console=console,
        )
        monitor = Confirm.ask(
            pad + "是否开启该账号的自动监控功能? (需要高级账号)", default=False, console=console
        )
        messager = Confirm.ask(
            pad + "是否开启该账号的自动水群功能? (需要高级账号)", default=False, console=console
        )
        telegram_accounts.append(TelegramAccount(phone=phone, monitor=monitor, messager=messager))
    if telegram_accounts:
        logger.info(f"即将尝试登录各账户并存储凭据, 请耐心等待.")
        await convert_session(telegram_accounts)

    emby_accounts = cfg.emby.account
    while True:
        if len(emby_accounts) > 0:
            logger.info(f"您当前填写了 {len(emby_accounts)} 个 Emby 账号信息.")
            more = Confirm.ask(pad + "是否继续添加?", default=False, console=console)
        else:
            more = Confirm.ask(pad + "是否添加 Emby 账号?", default=True, console=console)
        if not more:
            break
        url = Prompt.ask(
            pad + "请输入您的 Emby 站点 URL [dark_green](https://abc.com:443)[/]", console=console
        )
        username = Prompt.ask(pad + "请输入您在该 Emby 站点的用户名", console=console)
        password = Prompt.ask(
            pad + "请输入您在该 Emby 站点的密码 (不显示, 按回车确认)",
            password=True,
            console=console,
        )
        while True:
            time = Prompt.ask(
                pad + "设置模拟观看时长范围 (秒), 用空格分隔",
                default="120 240",
                show_default=True,
                console=console,
            )
            if " " in time:
                try:
                    time = [int(t) for t in time.split(None, 1)]
                    break
                except ValueError:
                    logger.warning(f"时长设置不正确, 请重新输入.")
            else:
                try:
                    time = int(time)
                    break
                except ValueError:
                    logger.warning(f"时长设置不正确, 请重新输入.")
        emby_accounts.append(EmbyAccount(url=url, username=username, password=password, time=time))
    advanced = Confirm.ask(pad + "是否配置高级设置", default=False, console=console)
    if advanced:
        while True:
            logger.info("发送关键日志消息到以下哪个账户?")
            logger.info(f"\t0. 不使用消息推送功能")
            for i, t in enumerate(telegram_accounts):
                logger.info(f'\t{i+1}. {t["phone"]}')
            selected = IntPrompt.ask(pad + "请选择", default=1, console=console)
            if selected:
                if selected > 0:
                    if selected <= len(telegram_accounts):
                        cfg.notifier.enabled = True
                        cfg.notifier.account = selected
                        break
                    else:
                        logger.warning("选择的账号不存在, 请重新选择.")
                else:
                    cfg.notifier.enabled = False
                    break
            else:
                logger.warning("选择的账号不存在, 请重新选择.")
        cfg.checkiner.timeout = IntPrompt.ask(
            pad + "设置每个 Telegram Bot 签到的最大尝试时间 (秒)",
            default=cfg.checkiner.timeout,
            show_default=True,
            console=console,
        )
        cfg.checkiner.retries = IntPrompt.ask(
            pad + "设置每个 Telegram Bot 签到的最大尝试次数",
            default=cfg.checkiner.retries,
            show_default=True,
            console=console,
        )
        cfg.checkiner.concurrency = IntPrompt.ask(
            pad + "设置最大可同时进行的 Telegram Bot 签到",
            default=cfg.checkiner.concurrency,
            show_default=True,
            console=console,
        )
        cfg.checkiner.random_start = IntPrompt.ask(
            pad + "设置计划任务时, 各站点之间签到的随机时间差异 (分钟)",
            default=cfg.checkiner.random_start,
            show_default=True,
            console=console,
        )
    content = item(cfg.model_dump(exclude_none=True)).as_string().encode()
    content = base64.b64encode(content)
    logger.info(
        f"您的配置[green]已生成完毕[/]! 您需要将以下内容写入托管平台的 EK_CONFIG 环境变量 ([red]SECRET[/]), 否则配置将在重启后丢失."
    )
    console.print()
    console.rule("EK_CONFIG")
    console.print(content.decode())
    console.rule()
    console.print()
    start_now = Confirm.ask(pad + "是否立即启动?", default=True, console=console)
    if start_now:
        config.set(cfg)
        return True
    else:
        return False


async def prepare_config(env_config: str):
    from tomlkit import item

    cfg = ConfigManager.load_env_config(env_config)
    cfg = ConfigManager.validate_config(cfg)
    config.set(cfg)
    if not cfg:
        return False

    # Add MongoDB check
    if not not cfg.mongodb:
        logger.warning("未设置 MongoDB 连接, 所有缓存数据将在重启后遗失或重置.")

    to_login = []
    for a in cfg.telegram.account:
        if not a.session:
            to_login.append(a)
    if to_login:
        logger.info("即将尝试登陆各个账号.")
        await convert_session(to_login)
        content = item(cfg.model_dump(exclude_none=True)).as_string().encode()
        content = base64.b64encode(content)
        logger.info(
            f"您已登陆到 Telegram! 您需要将以下内容重新写入托管平台的 EK_CONFIG 环境变量 ([red]SECRET[/]), "
            "否则登陆状态将在重启后丢失."
        )
        console.print()
        console.rule("EK_CONFIG")
        console.print(content.decode())
        console.rule()
        console.print()
        start_now = Confirm.ask(pad + "是否立即启动?", default=True, console=console)
        if start_now:
            config.set(cfg)
            return True
        else:
            return False
    else:
        return True


async def public_entrypoint():
    env_config = os.environ.get(f"EK_CONFIG", None)
    if env_config:
        return await prepare_config(env_config)
    else:
        return await interactive_config()



from __future__ import annotations

import asyncio
from datetime import date, datetime, time
from pathlib import Path
from typing import TYPE_CHECKING, List, Tuple

from loguru import logger
from pyrogram.types import User
import yaml

from embykeeper import __name__ as __product__
from embykeeper.data import get_data
from embykeeper.utils import show_exception, truncate_str, distribute_numbers
from embykeeper.runinfo import RunContext, RunStatus
from embykeeper.config import config
from embykeeper.schema import TelegramAccount

from ..session import ClientsSession
from ..link import Link
from ..pyrogram import Client

if TYPE_CHECKING:
    from loguru import Logger

__ignore__ = True


class SmartMessager:
    """自动智能水群类."""

    name: str = None  # 水群器名称
    chat_name: str = None  # 群聊的名称
    default_messages: str = None  # 语言风格参考话术列表资源名
    additional_auth: List[str] = []  # 额外认证要求
    min_interval: int = None  # 预设两条消息间的最小间隔时间
    max_interval: int = None  # 预设两条消息间的最大间隔时间
    at: Tuple[time, time] = None  # 可发送的时间范围
    msg_per_day: int = 10  # 每天发送的消息数量
    min_msg_gap = 5  # 最小消息间隔

    site_last_message_time = None
    site_lock = asyncio.Lock()

    def __init__(
        self,
        account: TelegramAccount,
        me: User = None,
        context: RunContext = None,
        config: dict = None,
    ):
        """
        自动智能水群类.
        参数:
            account: 账号登录信息
            context: 运行时上下文
            me: 当前用户
            config: 当前水群器的特定配置
        """
        self.account = account
        self.ctx = context
        self.config = config
        self.me = me
        self.min_interval = config.get(
            "min_interval", config.get("interval", self.min_interval or 60)
        )  # 两条消息间的最小间隔时间
        self.max_interval = config.get("max_interval", self.max_interval)  # 两条消息间的最大间隔时间
        self.log = logger.bind(scheme="telemessager", name=self.name, username=me.name)
        self.timeline: List[int] = []  # 消息计划序列
        self.example_messages = []

    async def get_spec_path(self, spec):
        """下载话术文件对应的本地或云端文件."""
        if not Path(spec).exists():
            return await get_data(spec, caller=f"{self.name}水群")
        else:
            return spec

    async def _start(self):
        """自动水群器的入口函数的错误处理外壳."""
        try:
            return await self.start()
        except Exception as e:
            if config.nofail:
                self.log.warning(f"发生错误, 自动水群器将停止.")
                show_exception(e, regular=False)
                return False
            else:
                raise

    async def start(self):
        """自动水群器的入口函数."""
        self.ctx.start(RunStatus.INITIALIZING)
        async with ClientsSession([self.account]) as clients:
            async for _, tg in clients:
                if self.additional_auth:
                    for a in self.additional_auth:
                        if not await Link(tg).auth(a, log_func=self.log.info):
                            return False

            if self.max_interval and self.min_interval > self.max_interval:
                self.log.warning(f"发生错误: 最小间隔不应大于最大间隔, 自动水群将停止.")
                return False

            if not await self.init():
                self.log.warning(f"状态初始化失败, 自动水群将停止.")
                return False

            messages_spec = self.config.get("messages", self.default_messages)
            if messages_spec and (not isinstance(messages_spec, str)):
                self.log.warning(f"发生错误: 参考语言风格列表只能为字符串, 代表远端或本地文件.")
                return False

            if messages_spec:
                messages_file = await self.get_spec_path(messages_spec)
                with open(messages_file, "r") as f:
                    data = yaml.safe_load(f)
                    self.example_messages = data.get("messages", [])[:100]

            self.log.bind(username=tg.me.name).info(
                f"即将预测当前状态下应该发送的水群消息, 但不会实际发送, 仅用于测试."
            )

            await self.send(dummy=True)

        if self.at:
            start_time, end_time = self.at
        else:
            start_time = time(9, 0, 0)
            end_time = time(23, 0, 0)

        start_datetime = datetime.combine(date.today(), start_time)
        end_datetime = datetime.combine(date.today(), end_time)

        start_timestamp = start_datetime.timestamp()
        end_timestamp = end_datetime.timestamp()

        msg_per_day = self.config.get("msg_per_day", self.msg_per_day)

        self.timeline = distribute_numbers(
            start_timestamp, end_timestamp, msg_per_day, self.min_interval, self.max_interval
        )

        # 检查并调整早于当前时间的时间点到明天
        now_timestamp = datetime.now().timestamp()
        for i in range(len(self.timeline)):
            if self.timeline[i] < now_timestamp:
                self.timeline[i] += 86400

        self.timeline = sorted(self.timeline)

        self.ctx.start(RunStatus.RUNNING)
        if self.timeline:
            while True:
                dt = datetime.fromtimestamp(self.timeline[0])
                self.log.info(f"下一次发送将在 [blue]{dt.strftime('%m-%d %H:%M:%S')}[/] 进行.")
                sleep_time = max(self.timeline[0] - datetime.now().timestamp(), 0)
                await asyncio.sleep(sleep_time)
                await self.send()
                self.timeline.pop(0)
                if not self.timeline:
                    break

    async def init(self):
        """可重写的初始化函数, 返回 False 将视为初始化错误."""
        return True

    async def get_infer_prompt(self, tg: Client, log: Logger, time: datetime = None):
        chat = await tg.get_chat(self.chat_name)
        context = []
        i = 0
        async for msg in tg.get_chat_history(chat.id, limit=50):
            i += 1
            if self.min_msg_gap and msg.outgoing and i < self.min_msg_gap:
                log.info(f"低于发送消息间隔要求 ({i} < {self.min_msg_gap}), 将不发送消息.")
                return
            spec = []
            text = str(msg.caption or msg.text or "")
            spec.append(f"消息发送时间为 {msg.date}")
            if msg.photo:
                spec.append("包含一张照片")
            if msg.reply_to_message_id:
                rmsg = await tg.get_messages(chat.id, msg.reply_to_message_id)
                spec.append(f"回复了消息: {truncate_str(str(rmsg.caption or rmsg.text or ''), 60)}")
            spec = " ".join(spec)
            ctx = truncate_str(text, 180)
            if msg.from_user and msg.from_user.name:
                ctx = f"{msg.from_user.name}说: {ctx}"
            if spec:
                ctx += f" ({spec})"
            context.append(ctx)

        prompt = "我需要你在一个群聊中进行合理的回复."
        if self.example_messages:
            prompt += "\n该群聊的聊天风格类似于以下条目:\n\n"
            for msg in self.example_messages:
                prompt += f"- {msg}\n"
        if context:
            prompt += "\n该群聊最近的几条消息及其特征为 (最早到晚):\n\n"
            for ctx in list(reversed(context)):
                prompt += f"- {ctx}\n"
        prompt += "\n其他信息:\n\n"
        prompt += f"- 我的用户名: {tg.me.name}\n"
        prompt += f'- 当前时间: {(time or datetime.now()).strftime("%Y-%m-%d %H:%M:%S")}\n'
        use_prompt = self.config.get("prompt")
        if use_prompt:
            prompt += f"\n{use_prompt}"
        else:
            extra_prompt = self.config.get("extra_prompt")
            prompt += (
                "\n请根据以上的信息, 给出一个合理的回复, 要求:\n"
                "1. 回复必须简短, 不超过20字, 不能含有说明解释, 表情包, 或 emoji\n"
                "2. 回复必须符合群聊的语气和风格\n"
                "3. 回复必须自然, 不能太过刻意\n"
                "4. 回复必须是中文\n\n"
                "5. 如果其他人正在就某个问题进行讨论不便打断, 或你有不知道怎么回答的问题, 请输出: SKIP\n\n"
                "6. 如果已经有很长时间没有人说话, 请勿发送继续XX等语句, 此时请输出: SKIP\n\n"
                "7. 请更加偏重该群聊最近的几条消息, 如果存在近期的讨论, 加入讨论, 偏向于附和, 允许复读他人消息\n\n"
                "8. 请勿@其他人或呼喊其他人\n\n"
                "9. 输出内容请勿包含自己的用户名和冒号\n\n"
                "10. 输出内容请勿重复自己之前说过的话\n\n"
            )
            if extra_prompt:
                prompt += f"{extra_prompt}"
            prompt += "\n请直接输出你的回答:"
        return prompt

    async def send(self, dummy: bool = False):
        async with ClientsSession([self.account]) as clients:
            async for _, tg in clients:
                chat = await tg.get_chat(self.chat_name)
                log = self.log.bind(username=tg.me.name)

                prompt = await self.get_infer_prompt(tg, log)

                if not prompt:
                    return

                answer, _ = await Link(tg).infer(prompt)

                if answer:
                    if len(answer) > 50:
                        log.info(f"智能推测水群内容过长, 将不发送消息.")
                    elif "SKIP" in answer:
                        log.info(f"智能推测此时不应该水群, 将不发送消息.")
                    else:
                        if dummy:
                            log.info(
                                f'当前情况下在聊天 "{chat.name}" 中推断可发送水群内容为: [gray50]{truncate_str(answer, 20)}[/]'
                            )
                        else:
                            log.info(
                                f'即将在5秒后向聊天 "{chat.name}" 发送: [gray50]{truncate_str(answer, 20)}[/]'
                            )
                            await asyncio.sleep(5)
                            msg = await tg.send_message(chat.id, answer)
                            log.info(f'已向聊天 "{chat.name}" 发送: [gray50]{truncate_str(answer, 20)}[/]')
                            return msg
                else:
                    log.warning(f"智能推测水群内容失败, 将不发送消息.")



from pyrogram.types import Message
from pyrogram.errors import MessageIdInvalid

from . import BotCheckin

__ignore__ = True


class HKACheckin(BotCheckin):
    name = "HKA"
    bot_username = "hkaemby_bot"
    bot_checkin_cmd = ["/cancel", "/start"]
    bot_text_ignore = ["对话已关闭"]

    async def message_handler(self, client, message: Message):
        text = message.caption or message.text
        if "选择菜单" in text:
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k:
                    try:
                        await message.click(k)
                    except TimeoutError:
                        self.log.debug(f"点击签到按钮无响应, 可能按钮未正确处理点击回复. 一般来说不影响签到.")
                    except MessageIdInvalid:
                        pass
                    return
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()

        if message.text and "未找到绑定用户" in message.text:
            self.log.warning(f"签到失败: 账户错误.")
            return await self.fail()

        await super().message_handler(client, message)



import asyncio
import base64
import binascii
import os
from pathlib import Path
import re
from typing import Optional, Union

import tomli as tomllib
from loguru import logger
from watchfiles import awatch
from pydantic import ValidationError
from appdirs import user_data_dir

from .utils import ProxyBase, deep_update
from .schema import (
    Config,
    EmbyAccount,
    format_errors,
)
from . import __name__ as __product__

logger = logger.bind(scheme="config")


class ConfigManager(ProxyBase):
    __noproxy__ = (
        "windows",
        "public",
        "basedir",
        "_basedir",
        "_conf_file",
        "_cache",
        "_observer",
        "_callbacks",
    )

    def __init__(self, conf_file=None):
        self.windows = False
        self.public = False

        self._basedir = None
        self._conf_file = conf_file
        self._cache = None
        self._observer = None
        self._callbacks = {
            "change": {},  # key -> [callback_funcs]
            "list_change": {},  # key -> [callback_funcs]
        }

    @property
    def basedir(self):
        if not self._basedir:
            return Path(user_data_dir(__product__))
        else:
            return Path(self._basedir)

    @basedir.setter
    def basedir(self, value):
        self._basedir = Path(value)
        if not self._basedir.is_dir():
            self._basedir.mkdir(parents=True, exist_ok=True)

    @property
    def __subject__(self):
        if not self._cache:
            raise RuntimeError("config not loaded")
        return self._cache

    def on_change(self, key, callback):
        """Register a callback for when a config value changes"""
        if key not in self._callbacks["change"]:
            self._callbacks["change"][key] = []
        self._callbacks["change"][key].append(callback)
        return CallbackHandle(self._callbacks["change"][key], callback)

    def on_list_change(self, key, callback):
        """Register a callback for when items in a list change"""
        if key not in self._callbacks["list_change"]:
            self._callbacks["list_change"][key] = []
        self._callbacks["list_change"][key].append(callback)
        return CallbackHandle(self._callbacks["list_change"][key], callback)

    def _process_changes(self, old_config, new_config):
        """Process changes between old and new configs and trigger callbacks"""

        def get_value(config, key):
            try:
                for part in key.split("."):
                    config = getattr(config, part)
                return config
            except AttributeError:
                return None

        # Process changes and deletions
        for key in self._callbacks["change"]:
            old_val = get_value(old_config, key) if old_config else None
            new_val = get_value(new_config, key) if new_config else None

            if old_val != new_val:
                for callback in self._callbacks["change"][key]:
                    callback(old_val, new_val)

        # Process list changes
        for key in self._callbacks["list_change"]:
            old_list = get_value(old_config, key) if old_config else []
            new_list = get_value(new_config, key) if new_config else []

            if isinstance(old_list, (list, tuple)) and isinstance(new_list, (list, tuple)):
                # Compare items directly instead of using sets
                added = [item for item in new_list if item not in old_list]
                deleted = [item for item in old_list if item not in new_list]

                if added or deleted:
                    for callback in self._callbacks["list_change"][key]:
                        callback(added, deleted)

    def set(self, value: Union[dict, Config]):
        if isinstance(value, dict):
            value = self.validate_config(value)
        if value:
            old_config = self._cache
            self._cache = value
            self._conf_file = None
            self._process_changes(old_config, value)
            return True
        else:
            return False

    @staticmethod
    def generate_example_config():
        """生成配置文件骨架, 并填入生成的信息."""

        from tomlkit import document, nl, comment, item, dumps
        from tomlkit.items import InlineTable
        from faker import Faker
        from faker.providers import internet, profile

        from .telegram.dynamic import get_names
        from . import __version__, __url__

        fake = Faker()
        fake.add_provider(internet)
        fake.add_provider(profile)

        default_config = Config()
        default_emby_account = EmbyAccount(url="http://example.com", username="", password="")

        doc = document()
        doc.add(comment("这是一个配置文件范例."))
        doc.add(comment("所有账户信息为生成, 请填写您的账户信息."))
        doc.add(comment(f"查看帮助与详情: {__url__}#安装与使用"))
        doc.add(nl())

        doc.add(comment("=" * 80))
        doc.add(comment("Emby 保活相关设置"))
        doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#emby-子项"))
        doc.add(comment("=" * 80))
        c = item({})
        c.add(nl())
        c.add(
            comment(
                '每次进行进行 Emby 保活的当日时间范围, 可以为单个时间 ("8:00AM") 或时间范围 ("<8:00AM,10:00AM>"):'
            )
        )
        c["time_range"] = default_config.emby.time_range
        c.add(nl())
        c.add(comment("每隔几天进行 Emby 保活:"))
        c["interval_days"] = default_config.emby.interval_days
        c.add(nl())
        c.add(comment("最大可同时进行的站点数:"))
        c["concurrency"] = default_config.emby.concurrency
        c.add(nl())
        c.add(comment("=" * 80))
        c.add(comment("Emby 账号, 您可以重复该片段多次以增加多个账号."))
        c.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#emby-account-子项"))
        c.add(comment("=" * 80))
        c["account"] = [{}, {}]
        for i in range(2):
            a: InlineTable = c["account"][i]
            a.comment(f"第 {i + 1} 个账号")
            if not i:
                a.add(nl())
                a.add(comment("站点域名和端口:"))
            a["url"] = fake.url(["https"]).rstrip("/") + ":443"
            if not i:
                a.add(nl())
                a.add(comment("用户名和密码:"))
            a["username"] = fake.profile()["username"]
            a["password"] = fake.password()
            if not i:
                a.add(nl())
                a.add(comment("模拟观看的随机时长范围 (秒), 可以为单个数字 (120) 或时间范围 ([120, 240]):"))
            a["time"] = default_emby_account.time
            if not i:
                a.add(nl())
                a.add(comment("以下为进阶配置, 请取消注释 (删除左侧的 #) 以使用:"))
                a.add(nl())
                a.add(comment("该服务器是否为 Jellyfin 服务器:"))
                a.add(comment(item({"jellyfin": True}).as_string()))
                a.add(comment("每隔几天进行保活, 默认使用全局设置 emby.interval_days:"))
                a.add(comment(item({"interval_days": default_config.emby.interval_days}).as_string()))
                a.add(comment("每次进行保活的当日时间范围, 默认使用全局设置 emby.time_range:"))
                a.add(comment(item({"time_range": default_config.emby.time_range}).as_string()))
                a.add(comment("使用连续播放, 主要用于增加观看时长:"))
                a.add(comment(item({"continuous": True}).as_string()))
                a.add(
                    comment(
                        "无法获取视频长度时, 依然允许播放 (默认最大播放 10 分钟左右, 可能播放超出实际长度):"
                    )
                )
                a.add(comment(item({"allow_stream": True}).as_string()))
                a.add(comment("取消注释以不使用配置文件定义的代理进行连接"))
                a.add(comment(item({"use_proxy": False}).as_string()))
                a.add(comment("取消注释以禁用该账户"))
                a.add(comment(item({"enabled": False}).as_string()))
        doc["emby"] = c
        doc.add(nl())

        doc.add(comment("=" * 80))
        doc.add(comment("Telegram 机器人签到相关设置"))
        doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#checkiner-子项"))
        doc.add(comment("=" * 80))
        c = item({})
        c.add(nl())
        c.add(
            comment(
                '每次进行进行 Telegram 签到的当日时间范围, 可以为单个时间 ("8:00AM") 或时间范围 ("<8:00AM,10:00AM>"):'
            )
        )
        c["time_range"] = default_config.checkiner.time_range
        c.add(nl())
        c.add(comment(":"))
        c["random"] = default_config.checkiner.random_start
        c.add(nl())
        c.add(comment("每个站点签到的最大超时时间:"))
        c["timeout"] = default_config.checkiner.timeout
        c.add(nl())
        c.add(comment("各站点最大可重试次数 (部分站点出于安全考虑有独立的设置):"))
        c["retries"] = 4
        c.add(nl())
        c.add(comment("最大可同时进行的站点数:"))
        c["concurrency"] = 1
        doc["checkiner"] = c
        c.add(nl())

        c.add(comment("=" * 80))
        c.add(comment("Telegram 账号, 您可以重复该片段多次以增加多个账号."))
        c.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#telegram-account-子项"))
        c.add(comment("=" * 80))
        c = item({"account": [{}, {}]})
        for i in range(2):
            a: InlineTable = c["account"][i]
            a.comment(f"第 {i + 1} 个账号")
            if not i:
                a.add(nl())
                a.add(comment('带国家区号的账户手机号, 一般为 "+86..."'))
            a["phone"] = f'+861{fake.numerify(text="##########")}'
            if not i:
                a.add(nl())
                a.add(comment("启用机器人签到系列功能, 默认启用, 设置为 false 以禁用:"))
            a["checkiner"] = True
            if not i:
                a.add(nl())
                a.add(
                    comment("启用群组监控系列功能, 包括抢邀请码和回答问题等, 默认禁用, 设置为 true 以启用:")
                )
            a["monitor"] = False
            if not i:
                a.add(nl())
                a.add(comment("启用自动水群系列功能, 风险较高, 默认禁用, 设置为 true 以启用:"))
            a["messager"] = False
            a.add(nl())
        doc["telegram"] = c

        doc.add(comment("=" * 80))
        doc.add(comment("站点相关设置"))
        doc.add(comment("当您需要禁用某些站点时, 请将该段取消注释并修改."))
        doc.add(comment(f"该部分内容是根据 {__product__.capitalize()} {__version__} 生成的."))
        doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#site-子项"))
        doc.add(comment("=" * 80))
        doc.add(nl())
        doc.add(comment(f'使用 "all" 代表所有签到器, "sgk" 以代表所有社工库签到器.'))
        doc.add(nl())
        doc.add(comment("案例 (启用所有站点, 除了社工库站点):"))
        site = item(
            {
                "site": {
                    "checkiner": ["all", "-sgk"],
                }
            }
        )
        for line in site.as_string().strip().split("\n"):
            doc.add(comment(line))
        doc.add(nl())
        doc.add(comment("案例 (启用默认站点, 额外增加 temby 站点):"))
        site = item(
            {
                "site": {
                    "checkiner": ["+temby"],
                }
            }
        )
        for line in site.as_string().strip().split("\n"):
            doc.add(comment(line))
        doc.add(nl())
        doc.add(comment("可以分别设置各个组件 (机器人签到 / 群组监控 / 自动水群) 的站点:"))
        site = item(
            {
                "site": {
                    "checkiner": ["-terminus", "-temby"],
                    "monitor": ["-misty"],
                    "messager": ["pornemby"],
                }
            }
        )
        for line in site.as_string().strip().split("\n"):
            doc.add(comment(line))
        doc.add(nl())
        site = item(
            {
                "site": {
                    "checkiner": get_names("checkiner"),
                    "monitor": get_names("monitor"),
                    "messager": get_names("messager"),
                }
            }
        )
        doc.add(comment(f"默认启用站点:"))
        for line in site.as_string().strip().split("\n"):
            doc.add(comment(line))
        doc.add(nl())
        site = item(
            {
                "site": {
                    "checkiner": get_names("checkiner", allow_ignore=True),
                    "monitor": get_names("monitor", allow_ignore=True),
                    "messager": get_names("messager", allow_ignore=True),
                }
            }
        )
        doc.add(comment(f"全部可用站点:"))
        for line in site.as_string().strip().split("\n"):
            doc.add(comment(line))
        doc.add(nl())

        doc.add(comment("=" * 80))
        doc.add(comment("代理相关设置"))
        doc.add(
            comment("代理设置, Emby 和 Telegram 均将通过此代理连接, 服务器位于国内时请配置代理并取消注释")
        )
        doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#proxy-子项"))
        doc.add(comment("=" * 80))
        doc.add(nl())
        proxy = item(
            {
                "proxy": {
                    "hostname": "127.0.0.1",
                    "port": 1080,
                    "scheme": "socks5",
                }
            }
        )
        proxy["proxy"]["scheme"].comment("可选: http / socks5")
        for line in proxy.as_string().strip().split("\n"):
            doc.add(comment(line))
        doc.add(nl())

        doc.add(comment("=" * 80))
        doc.add(comment("日志推送相关设置"))
        doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#notifier-子项"))
        doc.add(comment("=" * 80))
        c = item({})
        c.add(nl())
        c.add(comment("启用签到/保活结果的日志推送:"))
        c["enabled"] = True
        c.add(comment("使用第几个 Telegram 账号进行推送, 从 1 开始计数:"))
        c["account"] = 1
        c.add(
            comment(
                "默认情况下, 日志推送将在每天指定时间统一推送 (在 @embykeeper_bot 设置), 设置为 false 以立刻推送"
            )
        )
        c["immediately"] = False
        c.add(comment("默认情况下, 启动时立刻执行的一次签到/保活不会推送消息, 设置为 true 以推送"))
        c["once"] = False
        doc["notifier"] = c
        doc.add(nl())

        doc.add(comment("=" * 80))
        doc.add(comment("Subsonic 保活相关设置 (包括 Navidrome 和其他支持 Subsonic API 的音乐服站点)"))
        doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#subsonic-子项"))
        doc.add(comment("=" * 80))
        c = item({})
        c.add(nl())
        c.add(
            comment(
                '每次进行进行 Subsonic 保活的当日时间范围, 可以为单个时间 ("8:00AM") 或时间范围 ("<8:00AM,10:00AM>"):'
            )
        )
        c["time_range"] = default_config.subsonic.time_range
        c.add(nl())
        c.add(comment("每隔几天进行 Subsonic 保活:"))
        c["interval_days"] = default_config.subsonic.interval_days
        c.add(nl())
        c.add(comment("最大可同时进行的站点数:"))
        c["concurrency"] = default_config.subsonic.concurrency
        c.add(nl())
        c.add(comment("=" * 80))
        c.add(comment("Subsonic 账号, 您可以重复该片段多次以增加多个账号."))
        c.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#subsonic-account-子项"))
        c.add(comment("=" * 80))
        c["account"] = [{}, {}]
        for i in range(2):
            a: InlineTable = c["account"][i]
            a.comment(f"第 {i + 1} 个账号")
            if not i:
                a.add(nl())
                a.add(comment("站点域名和端口:"))
            a["url"] = fake.url(["https"]).rstrip("/") + ":443"
            if not i:
                a.add(nl())
                a.add(comment("用户名和密码:"))
            a["username"] = fake.profile()["username"]
            a["password"] = fake.password()
            if not i:
                a.add(nl())
                a.add(comment("模拟观看的随机时长范围 (秒), 可以为单个数字 (120) 或时间范围 ([120, 240]):"))
            a["time"] = default_emby_account.time
            if not i:
                a.add(nl())
        doc["subsonic"] = c
        return dumps(doc)

    def reset(self):
        self._cache = None

    @staticmethod
    def validate_config(config: Optional[dict] = None):
        """验证配置文件格式"""
        if config is None:
            return None
        try:
            return Config(**config)
        except ValidationError as e:
            logger.error(format_errors(e))
            return None

    async def start_observer(self):
        async def observer():
            async for changes in awatch(self._conf_file):
                logger.info(f"配置文件已更改，正在重新加载.")
                await self.reload_conf(self._conf_file)

        if self._observer:
            self._observer.cancel()
            asyncio.gather(self._observer, return_exceptions=True)
        self._observer = asyncio.create_task(observer())

    @staticmethod
    def load_env_config(data: str):
        """从环境变量数据读入配置."""

        try:
            data = base64.b64decode(re.sub(r"\s+", "", data).encode())
        except binascii.Error:
            logger.error("环境变量 EK_CONFIG 定义的配置格式错误, 请调整并重试.")
            return None
        try:
            config = tomllib.loads(data.decode())
        except (tomllib.TOMLDecodeError, UnicodeDecodeError):
            logger.error("环境变量 EK_CONFIG 定义的配置格式错误, 请调整并重试.")
            return None
        else:
            logger.debug("您正在使用环境变量配置.")
        return config

    async def reload_conf(self, conf_file=None):
        """Load config from provided file or config.toml at cwd."""
        cfg_dict = {}
        env_config = os.environ.get(f"EK_CONFIG", None)
        if env_config:
            cfg_dict.update(self.load_env_config(env_config))
        else:
            if self.windows:
                default_conf_file = self.basedir / "config.toml"
            else:
                default_conf_file = Path("config.toml")
            if conf_file:
                conf_file = Path(conf_file)
            elif self._conf_file:
                conf_file = Path(self._conf_file)
            elif default_conf_file.is_file():
                conf_file = default_conf_file
            if conf_file:
                if conf_file.suffix.lower() == ".toml":
                    with open(conf_file, "rb") as f:
                        deep_update(cfg_dict, tomllib.load(f))
                else:
                    logger.error(f'配置文件 "{conf_file}" 不是 TOML 格式的配置文件.')
                    return False
            else:
                try:
                    with open(default_conf_file, "w+", encoding="utf-8") as f:
                        f.write(self.generate_example_config())
                except OSError as e:
                    logger.error(
                        f'无法写入默认配置文件 "{default_conf_file}", 请确认是否有权限进行该目录写入: {e}.'
                    )
                    return False
                if not self.windows:
                    logger.warning("需要一个 TOML 格式的配置文件.")
                    logger.warning(f'您可以根据生成的参考配置文件 "{default_conf_file}" 进行配置')
                return False

        cfg_model = self.validate_config(cfg_dict)
        if not cfg_model:
            return False

        if conf_file:
            logger.debug(f"现在使用的配置文件为: {conf_file.absolute()}")
            self.set(cfg_model)
            if not self._conf_file == conf_file:
                self._conf_file = conf_file
                await self.start_observer()
            return True


class CallbackHandle:
    def __init__(self, callback_list, callback):
        self._callback_list = callback_list
        self._callback = callback

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._callback in self._callback_list:
            self._callback_list.remove(self._callback)


config: Union[Config, ConfigManager] = ConfigManager()

if __name__ == "__main__":
    print(config.generate_example_config())



import asyncio
import random

from pyrogram.types import Message
from pyrogram.errors import BadRequest, MessageIdInvalid

from . import AnswerBotCheckin


class PornembyCheckin(AnswerBotCheckin):
    name = "Pornemby"
    bot_username = "Porn_Emby_Bot"
    bot_success_pat = r".*?(\d+)$"

    async def on_photo(self, message: Message):
        await asyncio.sleep(random.uniform(2, 4))
        async with self.client.catch_reply(self.bot_username) as f:
            try:
                await message.click("点击签到")
            except (TimeoutError, MessageIdInvalid):
                pass
            except BadRequest:
                self.log.warning(f"签到失败: 账户错误.")
                await self.fail()
            try:
                m = await asyncio.wait_for(f, 10)
            except asyncio.TimeoutError:
                self.log.warning(f"签到失败: 签到无回应, 您可能还没有注册 {self.name} Emby 账号.")
                await self.fail()
            else:
                return await self.on_text(m, m.text)



import random

from embykeeper.utils import to_iterable

from . import BotCheckin

__ignore__ = True


class TestGroupCheckin(BotCheckin):
    name = "群组签到测试"
    chat_name = "api_group"
    bot_checkin_cmd = "签到"

    async def send_checkin(self, **kw):
        cmd = random.choice(to_iterable(self.bot_checkin_cmd))
        await self.send(cmd)
        await self.finish(message="群组发言已发送")
